/*  A serial bootloader according to Atmel Corporation application note AVR109
 *
 *  Copyright (c) 2013, 2014  Albrecht Weinert, Bochum
 *
 *  Notwithstanding ample modifications and adaption
 *  by A. Weinert   < a-weinert.de >
 *  this serial bootloader still contains small parts of Atmel's original
 *  software, namely Revision 1.7, last updated June 07 2000 by "raapeland".
 *  So Atmel's original copyright and license conditions are valid, also.
 */

/**
 * \addtogroup weAutSys
 * @{
 * \addtogroup bootSup
 * @{
 *
 * \file boot109.c
 *
 * Implementation of \ref intro_sec "weAutSys"' serial bootloader program
 *
 * \ref intro_sec "weAutSys" comes with a \ref bootSup "serial bootloader"
 * according to Atmel Corporation application note AVR109
 * on "Self-programming".
 *
 * This bootloader is (as of end 2014) implemented for and tested on four
 * different target platforms respectively Controllers:
   -  ATmega1284P as used on the \ref intro_secH "weAut_01"
      automation module with runtime \ref intro_sec "weAutSys'"
   -  ATmega2560 on ArduinoMega2560 or on ArduinoMegaADK etc.
   -  ATmega328p as used on ArduinoUno
   -  ATmega32 as shipped e.g. with easyAVR V.7 and BoAVR2013
      evaluation boards
 *
 * By C macro control and by Make support this implementation covers
 * said platforms respectively Controllers.
 *
 * Besides to those targets this serial bootloader can easily be adapted to
 * all similar platforms respectively to all AVR 8 bit ATmega Cs featuring
 * self-programming capability.
 *
 * In this implementation the serial bootloader's baudrate is defaults to
 * 38400. This setting yields quite a good performance as well as good
 * transmission quality. In a way 38400 is the best value for serial
 * boot-loading. Using lower rates would throw performance away, while
 * higher baud-rates might require more buffering as flashing would get much
 * slower than serial transmission in some cases. <br />
 * Regarding the transmission quality 115200 was tested successfully on (real)
 * RS232 links and with excellent cabling conditions and very accurate
 * clock frequency generators.
 *
 * An exception to the 38400 baud rule is the easyAVR evaluation board. It is
 * shipped with a 8 MHz quartz for the CPU clock generator, even if the
 * ATmega32 could use 16 MHz. 8 MHz gives poor accuracy for 38400 baud; hence
 * we use 19200 as standard (set in the respective includes) for this platform.
 *
 * All target platform's specific settings are organised in include files for
 * both the C-Compiler and the make-tool. Please read those files for the
 * respective platform and on how to extend this project to other hardware
 * targets.
 *
 * For technical background information on serial bootloading for ATmegas and
 * this bootloader please read
 <a href="http://a-weinert.de/pub/AVRserBootl.pdf">a-weinert.de/pub/AVRserBootl.pdf</a>.
 *
 * 1
 * 25.01.2017
 */
/** @} */ // Doxygen addtogroup bootSup
/** @} */ // Doxygen addtogroup weAutSys

#include "we-aut_sys/ll_common.h"
#include "bootloader/boot109.h"

#include <avr/boot.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

/*  The bootloader's revision and date
 *
 *  The text starts with a blank and ends with one linefeed.
 *
 *  Hint to maintainer/programmer:   <br />
 *  a) The content may change (best according to SVN data) but never ever
 *     the length. <br />
 *  b) Keep in accordance with the latest respectively next to commit) SVN
 *     Revision of any project's make or source files. This file boot109.c
 *     for example is just <br />
 *  Rev.:                 1   <br />
 *  Date:          25.01.2017
 *  c) As an exception to rule b) you may decide not to change the content
 *     if your changes are purely cosmetic (concerning comments only). <br />
 *     Anyway do check if your last change did not alter the generated code
 *     and it's linkage. Criterion is: After building the bootloaders for all
 *     platforms supported none of the files boot109_$(PLATFORM)_ext.tab did
 *     change. In that case do make or change a note like e.g.: <br />
 *     All bootloaders from Rev.23 (20.11.2014) to  Rev.29 (16.12.2014)
 *     are equal both functionally and regarding linkage.
 */
// fixed length                    "keep-always-this-length\n" //
INFLASH(char const bootRevDat[]) = " 028 (2014-12-16 Tu)   \n";


#if defined(GET_LOCK_BITS)    // avr-libc >= 1.2.5
#  define _GET_LOCK_BITS() boot_lock_fuse_bits_get(GET_LOCK_BITS)
#  define _GET_LOW_FUSES() boot_lock_fuse_bits_get(GET_LOW_FUSE_BITS)
#  define _GET_HIGH_FUSES() boot_lock_fuse_bits_get(GET_HIGH_FUSE_BITS)
#  define _GET_EXTENDED_FUSES() boot_lock_fuse_bits_get(GET_EXTENDED_FUSE_BITS)
#endif // defined(GET_LOCK_BITS)

/*  Send the greeting lines for bootloader's start
 *
 *  This function sends some lines of text over the bootloader's serial link.
 *  These texts  (\ref bootloaderWlc, \ref bootloaderPlatf,
 *  \ref bootRevDat etc.) are to introduce the serial bootloader.
 *
 *  \sa sendSerBytes_P
 *  \sa sendSerBytes
 */
void bootLoaderGreet(void){
   sendSerBytes_P(FAR_ADD(bootloaderWlc));    // welcome text
   sendSerBytes_P(FAR_ADD(bootloaderPlatf));  // platform information
   sendSerBytes_P(FAR_ADD(bootTextRevis));    // revision prefix text
   sendSerBytes_P(FAR_ADD(bootRevDat));       // revision, see above
} // bootLoaderGreet()

/*  Software identifier/programmer signature */
INFLASH(char const programmerSign[]) = PROG_SIGN;

/*  The programmers software version  */
INFLASH(char const programmerSWver[]) = PROG_SW_VER;

/*  The programmers hardware version  */
INFLASH(char const programmerHWver[]) = PROG_HW_VER;


/** Greeting text for the case of being entered by CLI
 *
 *  If the bootloader was entered by a command line interpreter (CLI) due
 *  to human command via UART, a greeting response may be sent via that
 *  serial line extended by below hints on (AVR109) bootloader commands.
 *
 *  This response will only be sent if the macro
 *  ALLOW_UART_GREET_BOOTLOADER_BY_CLI is defined with a value !=0. This
 *  setting if usually done in a platform specific include file.
 */
INFLASH(char const greetByCLI[])     =
  "  \n Bootloader entered by (CLI) command "
    "\n Send p V v and S to enable flashing \n"
    "\n Send (capital!) E to leave \n";

/* Initialise system resources
 *
 *  This function may be called after reset or before a system
 *  re-initialisation, e.g. before entering or leaving the bootloader
 *  or the application software.
 *
 *  This function essentially disables all interrupts and initialises
 *  the platform's ports and other resources.
 */
void basicSystemInit(){
   wdt_disable();  cli();        // disable watchdog for initialisation
   asm volatile ("clr __zero_reg__");
   SPCR =  0; // turn off SPI for (re-)init (essential)

#if defined(BASIC_INIT)
   BASIC_INIT(); // defined by config.h in platform specific include
#else
#  warning "basic BASIC_INIT() for PLATFORM undefined"
#endif

#if TIMEOUT_LIGHTSHOW == 3
   toHMI8LEDchain(0); // reset DO_DI
#endif
} // basicSystemInit()


/* Internal helper: wait for UART receive with timeout and LED display
 *
 * Change history:
 * Rev. ..4 2014-10-16  : public in bootLib.c and linkable via _ext.tab
 * Rev. 14 2014-10-17.. : made private here w/o parameter checks;
 *                        may be in-lined as well
 *  \return 0: a byte was received; not 0: timeout
 *  \param sec: 1..72 timeout in seconds
 */
uint8_t waitForUARTrec(uint8_t sec){
#if TIMEOUT_LIGHTSHOW
   ADDR_T patPoint = FAR_ADD(timOutCountPatr[72 - sec]); // pattern pointer
#endif
   while (waitSerByte(5)) {
      setTheLed(0);
      if (! waitSerByte(5)) return 0;
#if   TIMEOUT_LIGHTSHOW
      sec = getFlashByte(patPoint++);
      toHMI8LEDchain(sec);
#else
      --sec;
#endif
      if (!sec) return 1; // time out while waiting for char
      setTheLed(1);
   } // wait for sec s timeout
   return 0; // received
} // waitForUARTrec(uint8_t)


/** The bootloader's start
 *
 *  This function is the bootloader's program entry. It contains the full
 *  (protocol AVR109) state machine.
 *
 * \internal
 *  All (state) variables are local to this function.
 *  This AVR109 bootloader implementation uses no static RAM variables.
 *  Everything (all state) is kept on function local variables (i.e. stack).
 *  Rationale: This procedere avoids collisions with application software
 *             linked to and using bootloader functions.
 * \endinternal
 * \return main returns nothing on an embedded Controller;
 *         declaring an int function is just required by C tradition
 */
int main(void) __attribute__((OS_main));  // OS_main is like ((noreturn))
int main(void){
   cli();
   uint8_t resetCauses = MCUSR; // saved reset cause(s)
   basicSystemInit();
   initUART0(STD_BAUD, OFF, 0, 1, OFF); // initialise UART
   if ((resetCauses & 8) && !isFlashCleared()) { // watch dog reset
     appMain(OFF); // to application without re-do basicSystemInit()
   }  // watch dog reset: back to application with MCUSR unchanged

   uint8_t time_out_rec_set = 72;  // timeout > minute (pattern 0x80)

   if (isFlashCleared()) { // application flash empty: may use UART to humans
     bootLoaderGreet();
     sendSerBytes_P(FAR_ADD(bootTextReset));
     sendSerBytes_P(resetCauseText_P(resetCauses));
     sendSerBytes_P(FAR_ADD(greetEmptFlash));
     recvErrorState(); // flush receiver and errors
     MCUSR = 0; // clear reset cause
     goto startAVR109prot;
   } else { // else application flash not empty: probably application present
     // normally do not send any text via serial link
// #define TEST_GREET_BOOTLOADER_ON_APLLICATION_PRESENT 1
#if TEST_GREET_BOOTLOADER_ON_APLLICATION_PRESENT
     bootLoaderGreet();         // should not use UART before application or
     sendSerBytes_P(FAR_ADD(bootTextReset));         // bootloader operation
     sendSerBytes_P(resetCauseText_P(resetCauses));  // EXCEPT for TEST (1)
     recvErrorState(); // flush receiver and errors
#endif
   } // else non empty flash

   if ((resetCauses & 2) && hwCritEntBoot()) { // external reset and ext.crit
      goto startAVR109prot; // HW criterion to enter programming mode
   } // external reset and extra criterion (can be "enter" key on weAut_01)

   if( !resetCauses) { // no reset cause must be CLI command
#if ALLOW_UART_GREET_BOOTLOADER_BY_CLI
      bootLoaderGreet();
      sendSerBytes_P(FAR_ADD(greetByCLI));
      recvErrorState(); // flush receiver and errors
#endif
      goto startAVR109prot;
   } // no reset cause must be CLI command (came w/o reset from application)

   // to here by reset or power-on with application already flashed
   time_out_rec_set = 4;  // timeout just 4s; wait for avrDude's ESC

   startAVR109prot: {} // enter programming mode / start of bootLoader

   ADDR_U address;       // holds 16 or 17 .. 18 bit address
   uint8_t loByteHold = 0; // hold lo data byte (from c to C)

   // AVRdude starts communication by commands  Esc S V v p ...
   // This status variable is kept !=0 (true) as long as we haven't
   // seen a complete avrDude prelude. When true no command changing
   // the program memory (flash) will be accepted. This feature, of
   // course, is to protect an application program from random serial
   // communication after a reset or power up.
   uint8_t noAVRdudeCom = 0x1F;  // ... clearing Bits 4..0 (by Esc S V...)

   for(;;) { // endless loop until end command (E) or timeout

      if (waitForUARTrec(time_out_rec_set)) { // timeout while waiting
         if (isFlashCleared()) {
            time_out_rec_set = SEC_with_FF; // timeout start with pattern FF
            recvErrorState(); // flush receiver and errors
            continue; // we have no program ==> must stay in bootloader
         } else {    //
            break;  // out of boot loader to normal program
         }
      } // timeout while waiting for UART byte

      setTheLed(ON); // flicker (and test) on when receiving  (250ns)
      uint8_t val = getSerByte(); // get command character.
      setTheLed(OFF); // flicker (and test) off when sending (250ns, when A...

      if (val == 'p') {  //---- Get programmer type
         sendSerByte('S'); // S = serial
         noAVRdudeCom &= ~0x01; // p clears Bit 0
         if (noAVRdudeCom == 0x10) noAVRdudeCom = 0; // clear ESC if last one
         continue;
      }
      if(val == 'V')  { // Return software version (always 2 digits)
         sendSerBytes_P(FAR_ADD(programmerSWver));
         noAVRdudeCom &= ~0x04; // V clears Bit 2
         if (noAVRdudeCom == 0x10) noAVRdudeCom = 0; // clear ESC if last one
         continue;
      } // V
      if(val == 'v')  { // Return hardware version (always 2 digits)
         sendSerBytes_P(FAR_ADD(programmerHWver));
         noAVRdudeCom &= ~0x02; // v clears Bit 1
         if (noAVRdudeCom == 0x10) noAVRdudeCom = 0; // clear ESC if last one
         continue;
      } // v
      if (val == 'S')  { // programmer identifier (always 7 characters)
         sendSerBytes_P(FAR_ADD(programmerSign));  // Software identifier
         noAVRdudeCom &= ~0x08; // S clears Bit 3
         if (noAVRdudeCom == 0x10) noAVRdudeCom = 0; // clear ESC if last one
         continue;
      } // S
      // -- end of (avrdude) start sequence commands pVvS
      if (val <= ' ') { // white space & control
         if (!resetCauses) { // ignore WS on human terminal
            if (noAVRdudeCom != 0x0F) { // not after E
               time_out_rec_set = 72;  // timeout > minute (pattern 0x80)
            }  // ignore WS on (may be still) human terminal
            continue;
         } // ignore WS on human terminal
         if (val == ESC) { // AVRdude sync. always OK
            noAVRdudeCom &= ~0x10; // Esc clears Bit 4
            time_out_rec_set = SEC_with_F8;  // timeout now 30 sec for next byte
         } else {
            sendSerByte('?'); // only Esc is OK for non human (AVRdude)
         }
         continue;
      } // white space & control

// ----    to here we should get only after full AVRdude start sequence   ----
      if (val == 'E') {  //---- Exit bootloader (always allowed)
         sendSerByte(CR);
         //  allow 7 seconds for next programmer (avrdude.exe) start or for
         time_out_rec_set = 7;  // switching the serial interface
         noAVRdudeCom = 0x0F; // reset AVRdude start sequence recogniser
         address.val = 0;
         continue;
      } // E

      if(val == 'A') { //---- Set address  (16 bit)
         // NOTE: Flash addresses are given in words, not bytes.
         address.val = 0;
         setTheLed(ON); // flicker (and test) on when receiving
         address.v8[1] = recvSerByte();
         address.v8[0] = recvSerByte();
         setTheLed(OFF); // flicker (and test) off when sending
         sendSerByte(CR); // AVR109 uses carriage return as OK answer
         continue;
      } //  A  ---- Set address  (16 bit)

      if(val == 'H') { //---- Set extended address  (24 bit)
         // NOTE: Flash addresses are given in words, not bytes.
         setTheLed(ON); // flicker (and test) on when receiving

#if LARGE_MEMORY
         address.v8[2] = recvSerByte();
         address.v8[3] = 0; // ignore all above 24 Bit for all devices
#else
         recvSerByte();  // ignore above 16 Bit for small devices
#endif
         address.v8[1] = recvSerByte();
         address.v8[0] = recvSerByte();
         setTheLed(OFF);   // flicker (and test) off when sending
         sendSerByte(CR); // AVR109 uses carriage return as OK answer
         continue;
      } //  H  ---- Set address  (24 bit)

      if(val == 'b')  {  //---- Check block load support
         sendSerByte('Y'); // yes we can
         sendSerByte((BLOCKSIZE >> 8) & 0xFF); // MSB first.
         sendSerByte(BLOCKSIZE & 0xFF); // Report BLOCKSIZE (bytes)
         continue;
      } //  b  ---- Check block load support

     // internal hint: command without possible harm may be (put) above here
     // e.g.: "g" does no harm and is a candidate. On the other hand,
     // "Start block read" is too complicated to make any sense outside an
     // established avrDude context.

      if (noAVRdudeCom) { // to get here w/o AVRdude start is a serious error
         if (noAVRdudeCom & 0x10) { // no Escape or no full prelude yet
            if (val & 0xF8) { // may be spurious spike misinterpreted as start
               continue;
            } // may be spurious spike misinterpreted as start bit
         }  // no Escape or no full AVRdude prelude yet (may be spurious peak)
         if (!isFlashCleared()) break; // application is meant -- hopefully
         noAVRdudeCom = 0x1F; // reset AVRdude start sequence recogniser
         continue;  // no application keep on
      } // probably no communication with an AVRdude programmer tool

      // internal hint: all commands possibly doing harm must be below

      if(val == 'g') { //---- Start block read
         wait25();
         setTheLed(ON); // flicker (and test) on when receiving
         ucnt16_t blkSz;
         blkSz.v8[1] = recvSerByte();  // blocksize Hi
         blkSz.v8[0] = recvSerByte();  // blocksize Lo
         uint8_t recv = recvSerByte(); // get memtype
         setTheLed(OFF); // flicker (and test) off when sending
         if (recv == 'E') {
             address.val = blockReadEE(blkSz.val, address.val);
          } else if (recv == 'F') {
             address.val = blockReadFl(blkSz.val, address.val);
          } else {
             sendSerByte('?'); // only E or F memory type
          }

      } else if(val == 'B') { //---- Start block load
         wait25();
         setTheLed(ON); // flicker (and test) on when receiving
         ucnt16_t blkSz;
         blkSz.v8[1] = recvSerByte();  // blocksize Hi
         blkSz.v8[0] = recvSerByte();  // blocksize Lo
         uint8_t recv = recvSerByte(); // get memtype
         setTheLed(OFF); // flicker (and test) off when sending
         if (recv == 'E') {
            address.val = blockLoadEE(blkSz.val, address.val);
            sendSerByte( CR );
         } else if (recv == 'F') {
            address.val = blockLoadFl(blkSz.val, address.val);
            sendSerByte( CR );
         } else {
            sendSerByte('?'); // only E or F memory type
         }
      } else if(val == 'e') { //---- Chip erase
        for(address.val = 0; address.val < APP_END;
                                              address.val += SPM_PAGESIZE) {
            uint8_t tL =  address.v8[1]; // (address & 0x0FF00) >> 8;
            setTheLed(tL & 0x08); // flicker a bit faster than 100 ms
            _WAIT_FOR_SPM();
            _PAGE_ERASE(address.val); // this is a byte not a word address
         } // for
         sendSerByte(CR); // AVR109 OK

       } else if(val == 'R') {   //--- Read program memory
         // Send high byte, then low byte of flash word.
         _WAIT_FOR_SPM();
         _ENABLE_RWW_SECTION();
         ADDR_T addr = address.val << 1; // make byte address
         sendSerByte( getFlashByte(addr + 1));
         sendSerByte( getFlashByte(addr));
         ++address.val; // Auto-advance to next Flash word

      } else if(val == 'c') { //----  Write program memory, low byte.
         wait25();
         setTheLed(ON); // flicker (and test) on when receiving
         // NOTE: Always use this command before sending high byte.
         loByteHold = recvSerByte(); // Get low byte for later _FILL_TEMP_WORD.
         setTheLed(OFF); // flicker (and test) off when sending
         sendSerByte(CR); // Send OK back.

      } else if(val == 'C') { //---- Write program memory, high byte
         wait25();
         setTheLed(ON); // flicker (and test) on when receiving
         uint16_t temp_int = (recvSerByte() <<8 ) | loByteHold; // Get and insert high byte.
         _WAIT_FOR_SPM();
         _FILL_TEMP_WORD( (address.val << 1), temp_int ); // Convert word-address to byte-address and fill.
         ++address.val; // Auto-advance to next Flash word.
         setTheLed(OFF); // flicker (and test) off when sending
         sendSerByte(CR); // Send OK back

      } else if(val== 'm') { //---- Write page.
         if( address.val >= (APP_END >> 1) ) { // Protect bootloader area
            sendSerByte('?');
         } else  {
            _WAIT_FOR_SPM();
            _PAGE_WRITE( address.val << 1 ); // Convert word-address to byte-address and write.
         }
         sendSerByte(CR); // Send OK back

      } else if (val == 'D') { //---- Write EEPROM memory
         _WAIT_FOR_SPM();
         EEARL = address.v8[0]; // Setup EEPROM address.
         EEARH = address.v8[1];
         EEDR = recvSerByte(); // Get byte.
         EECR |= (1<<EEMPE); // Write byte.
         EECR |= (1<<EEPE);
         while (EECR & (1<<EEPE)) ; // Wait for write operation to finish
         ++address.val; // Auto-advance to next EEPROM byte.
         sendSerByte(CR);// Send OK back.

      } else if (val == 'd') { //---- Read EEPROM memory
         EEARL = address.v8[0]; // Setup EEPROM address.
         EEARH = address.v8[1];
         EECR |= (1<<EERE); // Read byte...
         sendSerByte(EEDR); // ...and send it back.
         ++address.val; // Auto-advance to next EEPROM byte.

      } else if(val == 'l') { //---- Write lockbits
         _WAIT_FOR_SPM();
         _SET_LOCK_BITS( recvSerByte() ); // Read and set lock bits
         sendSerByte(CR); // Send OK
      }
#if defined(_GET_LOCK_BITS)
      else if (val == 'r') { //---- Read lock bits.
         _WAIT_FOR_SPM();
         sendSerByte( _GET_LOCK_BITS() );

      } else if (val == 'F') { //---- Read fuse bits
         _WAIT_FOR_SPM();
         sendSerByte( _GET_LOW_FUSES() );

      } else if(val == 'N') { //---- Read high fuse bits
         _WAIT_FOR_SPM();
         sendSerByte( _GET_HIGH_FUSES() );

      } else if(val == 'Q') { //---- Read extended fuse bits
         _WAIT_FOR_SPM();
         sendSerByte( _GET_EXTENDED_FUSES() );
      }
#endif /* defined(_GET_LOCK_BITS) */
      else if(val == 'a') {  //---- Check autoincrement status (yes, we can)
         sendSerByte('Y');
      }  else if(val == 'P' || val == 'L') { // Enter / leave programming mode
         sendSerByte(CR); // Nothing to do, just answer OK

      }  else  if (val == 't') {  //---- Return supported device codes.
         // Note: The whole partcode issue is outdated.
         sendSerByte( PARTCODE ); // Supports only this device, of course.
         sendSerByte( 0 ); // Send list terminator

      } else if (val == 'x') { //---- Set the LED
         recvSerByte(); // Ignore the  parameter.
         setTheLed(ON);  // at present useless, as we flicker it
         sendSerByte(CR); // Send OK

      } else if (val == 'y') { //---- Clear the LED
         recvSerByte(); // Ignore the  parameter
         setTheLed(OFF);  // at present useless, as we flicker it
         sendSerByte(CR); // Send OK back

      } else if (val == 'T') {  //--- Set device type
         recvSerByte(); // Ignore the command and it's parameter.
         sendSerByte(CR); // Send OK back.

      } else  if(val == 's') {  //---- Return signature bytes
         sendSerByte( SIGNATURE_2 );
         sendSerByte( SIGNATURE_1 );
         sendSerByte( SIGNATURE_0 ); // always 1E for Atmel

      } else { // no recognised command and no escape
         sendSerByte('?');  // ignore
      }  // no recognised command and no escape
   } // end of for(;;)
   appMain(ON); // to application with re-do basic initialisation to be sure
} // main()
