/**
 * \file config_arduMega2560.h
 *
 *  Configuration settings for ArduinoMega2560 evaluation board
 *     and ArduinoADK, too
 *
 *  This file contains some platform specific definitions concerning software
 *  and hardware configuration. These settings influence the compilation and
 *  build process. Most of those settings can't be changed later at runtime.
 *
 *  Platform: arduMega2560; CPU: ATmega2560;
 *  atmega2560   # takes avr\iom2560.h + iomxx0_1.h
 *
 *  Hardware specifica and settings:
 *
 *  Port A: DI channels     <br />
 *  Port L: DI LEDs         <br />
 *  Port B: 7= yeLED; 6= buzzer; 5= redLED;
 *          4= TestPin0; 3-2-1= ISP;set as input <br />
 *  Port H: 4= TestPin1;    <br />
 *  Port C: DO channnels      <br />
 *  Port F: usable as analogue input <br />
 *  Port K: usable as analogue input <br />
 *
 *
 * This file is part of weAutSys &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 * <br />It is also used for the \ref bootSup "serial bootloader".
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2014  Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * 1
 * 25.01.2017
 */
//  SVN last change by A. Weinert

/** @} */ // Doxygen addtogroup   comTypes
/** @} */ // Doxygen addtogroup appLaySup

#ifndef CONFIG_PLATFORM_H
# define CONFIG_PLATFORM_H arduMega2560
#else
# error "more than one platform specific config_platform.h file"
#  include <avr/iom2560.h>  // for documentation gen. only (never to here)
#endif

// {for Eclipse C AVR (indexer) bug only (horrible hack start) -----
#include <avr/io.h>
#include <avr\iom2560.h>
#include <avr\iomxx0_1.h>
// for Eclipse C AVR (indexer) bug only (horrible hack end) -----}


//--- platform specifica on ports ----------------------------------------

#define HAVE_NIC 0 //!< Network interface chip;  0: no switch \showinitializer
#define HAVE_SMC 0 //!< Small memory card;  0: no switch \showinitializer

#define HAVE_ETHERNET 0 //!< Ethernet available \showinitializer
#define HAVE_MODBUS 0 //!< Modbus on Ethernet available \showinitializer

#define UART0MAY_CTS 0  //!< UART0 may signal ability to receive by CTS \showinitializer
#define UART0MAY_RTS 0  //!< UART0 may use RTS to hold back sending \showinitializer
#define UART0SPIYKY 1 //!< UART0 has spiky USB2serial: filter FE,FF \showinitializer

#define GN_LED_PORT B //!< Port reserved for green Status LED \showinitializer
#define GN_LED_PIN 5 //!< Pin reserved for green Status LED \showinitializer
#define RD_LED_PORT B //!< Port for yellow Status LED \showinitializer
#define RD_LED_PIN 7 //!< Pin for yellow Status LED \showinitializer

#define BUZZER_PORT B //!< Port for buzzer (if any)  \showinitializer
#define BUZZER_PIN 6  //!< Pin for buzzer (if any) \showinitializer

// may be C but L is usually better
#define CHAIN8LED_PORT L //!< Port for 8 independent LEDs in a row  \showinitializer

/** Port initialisation
 *
 *  This macro does the basic port initialisation according to the platform's
 *  standard port usage. Anyway it has to cover the serial bootloader.
 */
#define BASIC_INIT() do { \
   DDRA  = 0;    /* basically all input */ \
   PORTA = 0xFF; /* pullups on  */         \
   DDRB = 0xF0;  /* 7: yeLED; 6: res. buzz; 5: res. rdLED; 4 testPin0 */  \
   PORTB = 0x8F; /* yeLED and pullups on */            \
   DDRC  = 0xFF; /* Port C DO or for 8 LEDs */ \
   DDRL  = 0xFF; /* Port L for 8 LEDs */ \
   PORTC = 0; PORTL = 0; \
   DDRD = 0;  /* basically all input */ \
   DDRE = 2; /* PE0 RxD0,  PE1 TxD0 (out), all other input */\
   PORTE = 3; /* RxD0, TxD0 1 resp. pull-up */ \
   DDRD = 0; DDRE = 0; DDRF = 0; \
   DDRG = 0;  DDRJ = 0;  \
   DDRK = 0;   \
   DDRH = 0x10; /* H4 = testPin 1  */ \
   PORTH = 0; /* testPin1 and pullups off */ \
} while(0)


/** Have and use a bootloader
 *
 *  This value determines the existence and usage of a bootloader:<br />
 *  0: no bootloader or at least no bootloader integration
 *  1: bootloader area used / bootloader existing
 *  2: bootloader fully integrated: application programme may use all
 *     No Read-While-Write (NRWW) flash section functions and constants.
 *
 *  Fully integrated (2) means that all Controller resets go through the
 *  bootloader. And the bootloader will not enter the program (=x00000)
 *  without having done all basic \ref BASIC_INIT() "initialisations", that
 *  hence can be omitted from system/application software. Full integration
 *  also means that the bootloader may be entered by (\ref shellCLI "CLI")
 *  command.
 *  \showinitializer
 */
#define USE_BOOTLOADER_PRESET 2

// no extra HW criterion (button) to enter / stay in bootloader
#define hwCritEntBoot() (0)

// Two bootloader settings (must either both or none be defined)
# define PROG_HW_VER "25" //!< The \ref programmerHWver \showinitializer
# define TIMEOUT_LIGHTSHOW 0 //!< Time out HMI display \showinitializer


/** Allow low grade crystal for CPU clock
 *
 * Arduinos (platforms arduMega2560 and arduinoUno) have so low grade crystals
 * that you either have to repair the (new) board or measure the real
 * frequency and define it as macro shitCrystal.
 *
 * If this macro ALLOW_shitCrystal is defined a value defined in shitCrystal
 * will be used in some places instead of the nominal F_CPU.
 * \showinitializer
 */
#define ALLOW_shitCrystal 1

# ifndef F_CPU
#  define F_CPU  16000000
# elif  F_CPU != 16000000
// warning only because of the low grade Arduino crystals
#  warning "non standard MCU frequency specified for platform (check)"
# endif
