/**
 * \file config_boAVR2013.h
 *
 *  Configuration settings for the boAVR2013 evaluation board
 *
 *  This an evaluation board for studend's work designed by
 *  Dipl.-Ing. Andreas Koch, Hochschule Bochum, in 2013.
 *
 *  This file contains some platform specific definitions concerning software
 *  and hardware configuration. These settings influence the compilation and
 *  build process. Most of those settings can't be changed later at runtime.
 *
 *  Platform: boAVR2013; CPU: ATmega32;
 *  atmega32     # takes avr\iom32.h
 *
 *  Hardware specifica and settings:
 *
 *  Port A: set as input (intended for plugable display; not used here) <br />
 *  Port B: all output for LEDs; 6=gn 3=rd 1,0=testpins(rd) <br />
 *  Port C: digital input with pullups on for low active buttons <br />
 *  Port D: 0,1 = R/TxD (UART); 5= Buzzer; <br />
 *
 *  Attention: a) To be used all Port B LEDs have to be connected
 *                via 8 jumpers.
 *             b) Those LEDs are (for whatever? / no good reason) VCC based
 *                and hence LO-active.
 *             Note the macros GN_LED_INV and RD_LED_INV to be defined as !
 *             and to be left undefined in all else (normal) cases.
 *
 * This file is part of weAutSys &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 * <br />It is also used for the \ref bootSup "serial bootloader".
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2014  Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * 1
 * 25.01.2017
 */
//  SVN last change by A. Weinert

/** @} */ // Doxygen addtogroup   comTypes
/** @} */ // Doxygen addtogroup appLaySup

#ifndef CONFIG_PLATFORM_H
# define CONFIG_PLATFORM_H boAVR2013
#else
# error "more than one platform specific config_platform.h file"
#endif

//--- platform check for Eclipse    ----------------------------------------
#ifndef __AVR_ATmega32__
# error ECLIPSE warning: Do set target to: atmega32   14745600Hz
#endif



//--- platform specifica on ports ----------------------------------------

#define HAVE_NIC 0 //!< Network interface chip;  0: no NIC \showinitializer
#define HAVE_SMC 0 //!< Small memory card;  0: no switch \showinitializer

#define HAVE_ETHERNET 0 //!< Ethernet available \showinitializer
#define HAVE_MODBUS 0 //!< Modbus on Ethernet available \showinitializer

#define UART0MAY_CTS 0  //!< UART0 may signal ability to receive by CTS \showinitializer
#define UART0MAY_RTS 0  //!< UART0 may use RTS to hold back sending \showinitializer
#define UART0SPIYKY 1 //!< UART0 has spiky USB2serial: filter FE,FF \showinitializer

#define GN_LED_PORT B //!< Port for yellow Status LED \showinitializer
#define GN_LED_PIN 6 //!< Pin for yellow Status LED \showinitializer
#define GN_LED_INV ! //!< yellow Status LED is low-active \showinitializer
#define RD_LED_PORT B //!< Port reserved for red Status LED \showinitializer
#define RD_LED_PIN 3 //!< Pin reserved for red Status LED \showinitializer
#define RD_LED_INV ! //!< yellow Status LED is low-active \showinitializer

#define BUZZER_PORT D //!< Port for buzzer (if any)  \showinitializer
#define BUZZER_PIN 5  //!< Pin for buzzer (if any) \showinitializer

#undef CHAIN8LED_PORT //!< No Port for 8 independent LEDs in a row  \showinitializer
#define NO_CHAIN8LED  //!< No 8 independent LEDs in a row  \showinitializer

/** Port initialisation
 *
 *  This macro does the basic port initialisation according to the platform's
 *  standard port usage. Anyway it has to cover the serial bootloader.
 */
#define BASIC_INIT() do { \
   DDRA  = 0;    /* basically all input (plugable display) */   \
   PORTA = 0xFF; /*  pullups on  */                              \
   DDRB = 0xFF;  /*  all output (LEDs) 4 Ye(gn), 3rd, 10 test */  \
   PORTB = ~0x19;/*  bootloader LEDs on; test1 off test0 on; lo-act! */ \
   DDRC  = 0x00; /* Port C all input for  8 button switches to 0 */ \
   PORTC = 0xFF; /* all pull-ups on for button switches  */          \
   DDRD = 0xFA;  /*  5 buzzer;   D1 TXD out */         \
   PORTD = 0x03; /* buzzer off; TxD  1; RxD  pull-up */ \
} while(0)

/** Report bootloader entrance by CLI on UART
 *
 *  Setting this to 1 we assume the UART on this platform being used for
 *  textual human interaction including command line interpreter (CLI).
 *  \showinitializer
 */
#define ALLOW_UART_GREET_BOOTLOADER_BY_CLI 1

/** Have and use a bootloader
 *
 *  This value determines the existence and usage of a bootloader:<br />
 *  0: no bootloader or at least no bootloader integration
 *  1: bootloader area used / bootloader existing
 *  2: bootloader fully integrated, using all No Read-While-Write (NRWW)
 *     flash section functions and constants applicable.
 *
 *  Fully integrated (2) means that all Controller resets go through the
 *  bootloader. And the bootloader will not enter the program (=x00000)
 *  without having done all basic \ref BASIC_INIT() "initialisations", that
 *  hence can be omitted from system/application software. Full integration
 *  also means that the bootloader may be entered by (\ref shellCLI "CLI")
 *  command.
 *  \showinitializer
 */
#define USE_BOOTLOADER_PRESET 2

// no extra HW criterion (button) to enter / stay in bootloader
#define hwCritEntBoot() (0)

// Two bootloader settings (must either both or none be defined)
#define PROG_HW_VER "13" //!< The \ref programmerHWver \showinitializer
#define TIMEOUT_LIGHTSHOW 3 //!< Time out HMI display \showinitializer


// check boAVR2013 F_CPU
# ifndef F_CPU
#  define F_CPU 14745600
# elif  F_CPU != 14745600
// warning only because of the low grade Arduino crystals
#  warning "non standard MCU frequency specified for boAVR2013 (check)"
# endif


// AVR32 has other port and register names
// as well as reduced capabilities
// Hint: this part may be put to an extra .h file as soon as
// more platforms with low grade AVRs are being defined
#define UDR0 UDR
#define UCSR0A UCSRA
#define UCSR0B UCSRB
#define UCSR0C UCSRC
#define RXC0 RXC
#define RXEN0 RXEN
#define TXC0 TXC
#define TXCIE0 TXCIE
#define UBRR0 UBRRL
#define RXCIE0 RXCIE
#define TXEN0 TXEN
#define U2X0 U2X
#define UDRE0 UDRE

#define URSEL   7

#define EEMPE EEMWE
#define EEPE EEWE

/** Unable to XOR output pin (when defined)
 *
 *  ATmega32 won't xor output port by writing to PINx
 *  \showinitializer
 */
#define CANT_OUT2PIN 1


/** Allow low grade crystal for CPU clock
 *
 * Arduinos (platforms arduMega2560 and arduinoUno) have so low grade crystals
 * that you either have to repair the (new) board or measure the real
 * frequency and define it as macro shitCrystal.
 *
 * If this macro ALLOW_shitCrystal is defined a value defined in shitCrystal
 * will be used in some places instead of the nominal F_CPU.
 * \showinitializer
 */
#define ALLOW_shitCrystal 1
