/**
 * \file config_weAut_01.h
 *
 *  Configuration settings for the weAut_01 automation module
 *
 *  This file contains some platform specific definitions concerning software
 *  and hardware configuration. These settings influence the compilation and
 *  build process. Most of those settings can't be changed later at runtime.
 *
 *  Platform: weAut_01 (weAut_00); CPU: ATmega1284p;
 *
 *  Hardware specifica and settings:
 *
 *  Port A: DI or AI channels     <br />
 *  Port B
    B0 in   enter key                    in  pullup               1
    B1 in   RTS in / counter in / free port JP9 (default in)      1
    B2 out  LED gn (0 = on)                                       0
    B3 in   comparator LV (Lastspannung)                          0
    B4 out  LED rd (1 = on)                                       1
    B7,6,5  SPI1-Master: B7clk, B5mosi must be set as output      101
    Port B out 2 4 (LEDs) 5 7 SPI
    init SPI 1 : used for DI_LEDS and DO
    Enable   SPI,         Master,      Int   (SPR1,0=00 : clock /4)
    SPI 1 runs with 1/2 CPU clock, ie. 10MHz, needing 16 clocks for one byte

 *
 *  Port C:
    C2, C3, C6: /CS=Clk : out 1;
    C5        : DO enab : out 0;           0: DO disable (LED red)
    C7   : ether /CS    : out 1;           ether /reset (out 0) on weAut_00
    C4        : DO OK   : in  1 pullup
    C01  : Testpin 12   : 1 pullup
 *
 *  Port D:
 *  D7   NIC interrupt  : in pullup
    D6   CTS generator  : out (default)
    D5   free port on weAut_01 respectively SMC insertion switch
    D4 3 2  SPI 2 MOSI MISO Clk : 4&3 out
    D1 0   UART 0 TX RX : 1 out
 *
 * This file is part of weAutSys &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 * <br />It is also used for the \ref bootSup "serial bootloader".
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2014  Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * 1
 * 25.01.2017
 */
//  SVN last change by A. Weinert

/** @} */ // Doxygen addtogroup   comTypes
/** @} */ // Doxygen addtogroup appLaySup

#ifndef CONFIG_PLATFORM_H
# define CONFIG_PLATFORM_H weAut_01
# define weAut_ 1
#else
# error "more than one platform specific config_platform.h file"
#endif

#define USART_IN_APP_WITH_INT ON //!< UART uses interrupt in application \showinitializer

#define SMC_INSERT 0xD5 //!< SMC insert switch;  0: no switch \showinitializer

/** Report bootloader entrance by CLI on UART
 *
 *  Setting this to 1 we assume the UART on this platform being used for
 *  textual human interaction including command line interpreter (CLI).
 *  \showinitializer
 */
#define ALLOW_UART_GREET_BOOTLOADER_BY_CLI 1

/** SMC insert check by software thread
 *
 *  If evaluated to non zero the SMC thread checks the card insertion by
 *  software at otherwise idle times about every 100 ms. This is not
 *  necessary if a hardware \ref SMC_INSERT "insert switch" is available and
 *  properly configured.
 *
 *  The insert test by software is not recommended if every SMC command is
 *  logged by \ref DEBUG_SMC > 1.
 */
#define SMC_INS_POLL ! SMC_INSERT && DEBUG_SMC <=1

//--- platform check for Eclipse    ----------------------------------------
#if ! defined __AVR_ATmega1284P__
# warning ECLIPSE warning: set target to atmega1284p 20000000
#endif

//--- platform specifica on ports ----------------------------------------

#define HAVE_NIC 2 //!< Network interface chip \showinitializer
#define HAVE_SMC 2 //!< Small memory card via SPI \showinitializer
#define USE_SPI2 1 //!< Use SPI2 for NIC and SMC \showinitializer

#define HAVE_ETHERNET 1 //!< Ethernet available \showinitializer
#define HAVE_MODBUS 1 //!< Modbus on Ethernet available \showinitializer

#define UART0MAY_CTS 1  //!< UART0 may signal ability to receive by CTS \showinitializer
#define UART0MAY_RTS 1  //!< UART0 may use RTS to hold back sending \showinitializer
#define UART0SPIYKY 0 //!< UART0 has spiky USB2serial: filter FE,FF \showinitializer

#define GN_LED_PORT B //!< Port for green Status LED \showinitializer
#define GN_LED_PIN 2 //!< Pin for green Status LED \showinitializer
#define RD_LED_PORT B //!< Port for red Status LED \showinitializer
#define RD_LED_PIN 4 //!< Pin for red Status LED \showinitializer


#define BUZZER_PORT C //!< Port for buzzer (if any)  \showinitializer
#define BUZZER_PIN 1  //!< Pin for buzzer (if any) \showinitializer


/** Port initialisation
 *
 *  This macro does the basic port initialisation according to the platform's
 *  standard port usage. Anyway it has to cover the serial bootloader.
 */
#define BASIC_INIT() do { \
   DDRA  = 0;    /* basically all input for DI or AI */ \
   PORTA = 0x00; /* no pullups as weAut_01 has filters / dividers */  \
   DDRB = (1<<4) | (1<<2) | (1<<5) | (1<<7); ;  /* 4257 out */  \
   PORTB = 0x35; /* enter, RTS PulllUp, gn out, rd 0n MISO PullUp */ \
   DDRC  = (1<<2) | (1<<3) | (1<<5) | (1<<6) | (1<<7); \
   PORTC = ~(_BV(5) ); \
   DDRD = (1<<1) | (1<<3) | (1<<4) | (1<<6);  /*  D5 not out any more */ \
   PORTD = 0xA6; /* D 7 5 1 2 (16.08.2012 pullup f. mem card) */ \
   SPCR = (1 << SPE)  | (1 << MSTR)  /* no Int (1 << SPIE) */ \
          | ( 1 << CPHA)  | ( 1 << CPOL); /* CPHA=CPOL=1 for 74HCT595 */ \
   SPSR = (1 << SPI2X);  /* SPI2X  = ( clock / 4 ) * 2   */ \
} while(0)


/** Have and use a bootloader
 *
 *  This value determines the existence and usage of a bootloader:<br />
 *  0: no bootloader or at least no bootloader integration
 *  1: bootloader area used / bootloader existing
 *  2: bootloader fully integrated: application programme may use all
 *     No Read-While-Write (NRWW) flash section functions and constants.
 *
 *  Fully integrated (2) means that all Controller resets go through the
 *  bootloader. And the bootloader will not enter the program (=x00000)
 *  without having done all basic \ref BASIC_INIT() "initialisations", that
 *  hence can be omitted from system/application software. Full integration
 *  also means that the bootloader may be entered by (\ref shellCLI "CLI")
 *  command.
 *  \showinitializer
 */
#define USE_BOOTLOADER_PRESET 2

// use enter button for staying in bootloader
# define hwCritEntBoot() ( !(PINB & 1))

// Two bootloader settings (must either both or none be defined)
# define PROG_HW_VER "01" //!< The \ref programmerHWver \showinitializer
# define TIMEOUT_LIGHTSHOW 0 //!< Time out HMI display \showinitializer

# ifndef F_CPU
#  define F_CPU  20000000
# endif
