/**
 * \addtogroup weAutSys
 * @{
 * \addtogroup bootSup
 * @{
 *
 * \file boot109.h
 *
 * Definitions of \ref intro_sec "weAutSys"' bootloader and helper functions
 *
 * These functions are to be used in situations before or after a reset or
 * a system re-initialisation, that is without any threading support (or
 * interrupts). This condition is met
    - in early initialisation phases of  \ref intro_sec "weAutSys'" system
      or application software and
    - at all the time in the bootloader.
 *
 * Everything defined here is to be linked and put to the NMWR (no modify
 * while read) respectively bootloader section of the flash/program memory.
 * It is usable by normal system and application software through certain
 * linkage mechanisms described in
 * in  &nbsp; &nbsp;
 *  Albrecht Weinert<br /> &nbsp; &nbsp;
 *   <a href="http://a-weinert.de/pub/AVRserBootl.pdf"
 *    >A serial bootloader for weAut_01, ArduinoMega and akin</a><br />
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2015 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp;
 *                            <<a href="http://a-weinert.de">a-weinert.de</a>>
 *
 * 1
 * 25.01.2017
 *
 * History
 * Rev. 8..13 2014-10-16 : stable _ext.tab;
 *                         check this out for clients linked threreto
 * Rev. 14 2014-10-17 .. : library cleanup; separate "light show" from
 *                         serial I/O functions
 */

/** @} */ // Doxygen addtogroup bootSup

#ifndef BOOT109_H
#define BOOT109_H

#include <stdint.h>
#include <avr/pgmspace.h>

#include "we-aut_sys/ll_common.h"  // just to please Eclipse


/**
 * \addtogroup bootSup
 * @{
 * \addtogroup  bootIntegr
 * @{
 */

/** Initialise system resources
 *
 *  This function does the basic initialisations for the target hardware and
 *  runtime. It essentially disables all interrupts and initialises the ports
 *  and other resources except the UART (for that see \ref initUART0). <br />
 *  The situations to use are <ul>
 *  <li> after reset </li>
 *  <li> before a system re-initialisation </li>
 *  </ul>
 *  that is <ul>
 *  <li> before entering the bootloader </li>
 *  <li> before leaving the bootloader (to enter application software) </li>
 *  </ul>
 */
void basicSystemInit(void);


/** Output to a chain of eight HMI/visible LEDs
 *
 *  If the target hardware has a line of eight LEDs this function sets them
 *  according to parameter \c val. Otherwise nothing is done. <br />
 *  The pre-condition is these eight LEDs being independently controllable by
 *  software without any side-effects on I/O to an external process controlled
 *  by the target hardware.
 *
 *  With \ref intro_secH "weAut_01" these are the eight green digital input
 *  (DI) display LEDs. <br />
 *  With ArduinoMega and easyAVR &mdash; assuming above conditions to be met
 *  &mdash; it might be just one eight bit output port named by the
 *  macro CHAIN8LED_PORT.
 *  If no setting is made for a chain of 8 independent LEDs and the macro
 *  NO_CHAIN8LED is defined an implementation may just do nothing (without
 *  compile time warning).
 */
void toHMI8LEDchain(uint8_t val);

/** Jump to application program
 *
 *  First of all this function disables all interrupts and the watchdog and
 *  does a \ref basicSystemInit() if \c init is not 0. Parameter \c init
 *  should only be \ref OFF if a
 *  ref basicSystemInit "basic initialisation" has been done and those
 *  settings haven't been spoiled.
 *
 *  This function then \ref initUART0 "initialises" the UART (0) with or
 *  without interrupt usage depending on the platform.
 *
 *  With freshly set zeroReg and stackpointer this function will then go to
 *  the normal program's start in the flash memory. Hence it will never
 *  return to the caller.
 *
 *  \param init ON: do full \ref basicSystemInit() also
 *  \sa bootMain
 */
void appMain(uint8_t init) __attribute__((noreturn));

/** Jump to the bootloader
 *
 *  At first this function disables all interrupts and the watchdog.
 *
 *  With freshly set zeroReg and stackpointer this function will then go to
 *  the bootloader's start in the flash memory. Hence this function
 *  will never return to the caller.
 *  \sa appMain
 */
void bootMain(void) __attribute__((noreturn));

/** Initialise the serial input (UART0)
 *
 *  UART 0 is initialised in asynchronous mode for receiving and transmitting
 *  (full duplex). A recommended baudRate is 38400; it's fast enough for
 *  communication and even boot loading and still robust enough for critical
 *  or long lines.
 *
 *  If in doubt x2 should be \ref OFF. It is needed only for very high
 *  baudRates with respect to the CPU clock frequency (F_CPU). And some exotic
 *  baudrate F_CPU combinations get higher accuracy with x2 \ref ON.
 *
 *  \param baudRate the rate desired (usually 38400)
 *  \param x2 true: double speed (half receive samples)
 *  \param parity 0=false: none; odd , even value: odd, even
 *  \param stopBits 2: 2 stop bits, 1 [default]: 1 stop bit
 *  \sa uartSetBaudDivide
 *  \param useInt true: use UART interrupts
 */
void initUART0(uint32_t baudRate,  uint8_t x2,
                        uint8_t parity, uint8_t stopBits, uint8_t useInt);


/** Bootloader's welcome greeting and copyright notice
 *
 *  This is a four line text in (high!) flash memory plus three starting
 *  and two trailing empty lines. It is meant for display as well as embedded
 *  copyright notice.
 */
extern char const bootloaderWlc[];

/** Greeting for empty application flash
 *
 *  This is a one line text in (high!) flash memory plus two starting
 *  and one trailing empty lines. It says: <br />
 *  remain in bootloader: application flash empty
 */
extern char const greetEmptFlash[];

/** Bootloader's platform name and CPU frequency
 *
 *  This is a one line text in (high!) flash memory ending in a linefeed.
 */
extern char const bootloaderPlatf[];

/** The text "Revision:"
 *
 *  This is a text block in (high!) flash memory (0-terminated).
 */
extern char const bootTextRevis[];

/** The text "Reset by:"
 *
 *  This is a text block in (high!) flash memory (0-terminated).
 */
extern char const  bootTextReset[];

/** The reset cause text
 *
 *  According to the bits defined for the MCUSR register this function
 *  returns a (flash memory) pointer to \code
 *  " power on   ", " external reset ", " power (brown) out ",
 *  " watchdog reset    ", " jtag reset ", " no reset (by command?) "
 *   or " unknown cause ".\endcode
 *
 *  \param resetCauses The C's reset info as by MCUSR
 *  \return a 32 or 16 bit pointer (\ref ADDR_T) to a 0-terminated String
 *          in high (bootloader) flash memory
 */
ADDR_T resetCauseText_P(uint8_t resetCauses);

extern char const  bootResetCause1[]; //!< reset cause: power on
extern char const  bootResetCause2[]; //!< reset cause: external
extern char const  bootResetCause4[]; //!< reset cause: brown out
extern char const  bootResetCause8[]; //!< reset cause: watchdog
extern char const  bootResetCauseG[]; //!< reset cause: JTAG
/** no reset cause: exit from from active bootloader or by command */
extern char const  bootResetCause0[];

extern char const  bootResetCauseN[]; //!< reset cause: not known


/** A very basic delay function keeping the CPU busy for about 25s
 *
 *  Calling this function will keep the CPU busy for 500 cycles (two less for
 *  small flash devices) on a 20 MHz platform (\ref intro_secH "weAut_01").
 *  That will take 25s.
 *  On slower platforms with e.g. 16 MHz (Arduino) the number of cycles is, of
 *  course, proportionally reduced.
 */
void wait25(void);


/** Generate a far address for high flash memory items
 *
 *  avr-gcc is restricted (may say dumb) enough always to deliver 16 bit
 *  addresses respectively pointers for variables even if they reside by
 *  \ref INFLASH or PROGMEM in higher addresses. Note that the ATmega1284P
 *  e.g. has 128K flash memory and the ATmega2560 e.g. has 256Kbyte.
 *
 *  Hence if for example the boatloader flash array \ref  timOutCountPatr is
 *  finally at address  0x01e20c \ref  timOutCountPatr "&timOutCountPatr[1]"
 *  will (with avr-gcc) be  0x0e20d instead of 0x1e20d. <br />
 *  Using  \c FAR_ADD(timOutCountPatr[1]) supplies both the &-operator and
 *  the missing 0x10000 respectively 0x30000, correcting the error described.
 *
 *  For devices with less than 64KByte = 32KWords flash memory this macro's
 *  value is just \c &(varHF).
 *
 *  To get a 32 bit address for an item in the lowest 64kbyte of use the
 *  macro \ref LOW_ADD instead.<br />
 *  Note that both macros \ref FAR_ADD and \ref LOW_ADD cover all flash of
 *  an ATmega1284P but only half of the ATmega2560's flash.<br />
 *  Note: Hopefully in future, avg-gcc will automatically switch to 24bit
 *  pointers on larger devices and automatically handle them correctly. What,
 *  in the end, we have higher level languages and their compilers for? But,
 *  alas, as of end 2015 this is still not the case.
 *
 *  \param varHF a variable in the highest 64Kbyte flash memory block. This
 *           covers any bootloader space. <br />
 *           Do not supply a pointer; the &amp; (i.e. the address operator)
 *           will be applied to the parameter.
 */
#define FAR_ADD(varHF) ((FLASHEND & 0xF0000) | (uint16_t)&(varHF))

/** Generate a far address for low flash memory items
 *
 *  This macro provides a 16 or 32 bit address (\ref ADDR_T) for an item (a
 *  variable) in lowest 64Kbate segment of the memory. The upper 16 bits of
 *  the address provided this way are 0.
 *
 *  Use this macro to make a 16 or 32 bit address (\ref ADDR_T)  parameter
 *  for e.g. \ref sendSerBytes_P. Giving a (16 bit) pointer here would work
 *  by the (correct) automatic cast, also &mdash; but with a compiler warning.
 *
 *  \param varLF a variable in the lowest 64Kbyte flash memory
 *         block. This covers the (start of) application space. <br />
 *         Do not supply a pointer; the &amp; (i.e. the address operator)
 *         will be applied to the parameter.
 *  \sa FAR_ADD
 */
#define LOW_ADD(varLF) ((ADDR_T)((uint16_t)&(varLF)))


/** Copy a string from flash memory to RAM
 *
 *  This function copies a String <ul>
 *  <li> from flash memory pointed to by a 32 bit address \c src </li>
 *  <li> to a RAM buffer pointed to by (normal) pointer \c dst.</li></ul>
 *
 *  The function copies up to \c mxLen-1 characters from \c src to \c dst
 *  and appends a trailing 0. Returned is the address of the last character
 *  in \c dst modified (i.e. the 0's address).
 *
 *  Rationale: Besides a variety of functions to handle and output strings in
 *  RAM \ref intro_sec "weAutSys" features a second (sub-) set of functions
 *  to handle / output strings in flash memory, their names usually ending
 *  in  \c P or \c _P. Insofar these functions usually save the trouble of
 *  copying strings form flash to RAM (prior to output e.g.). But due to
 *  avr-gcc C pointer limitations the standard flash variable handling won't
 *  work above 64K byte, as is the case with bootloader variables on large
 *  flash devices. A third set of functions for those use cases can hardly be
 *  justified. Here the workaround is "copy to an intermediate RAM buffer and
 *  use RAM handling functions". That workaround is support by this function.
 *
 *  \param dst the destination to modify (in RAM, not null!)
 *  \param src the source (string) to copy from (in flash memory, not null!)
 *  \param mxLen the maximum number of characters to modify in \c dst
 *         including the trailing 0 appended. The maximum advance of the
 *         return value to parameter \c dest
 *  \return dst + number of characters modified; NULL only if \c dst is NULL
 *  \sa getSomeCharsP(char*, prog_char*, uint8_t)
 *  \sa FAR_ADD
 *  \sa LOW_ADD
 *  \sa resetCauseText_P()
 */
char * copyChars_P(char * dst, ADDR_T  src,  uint8_t mxLen);


/** Basic UART send one byte (guarded)
 *
 *  This is a very basic send function for the \ref initUART0 "initialised"
 *  UART(0). It will wait for the UART being ready to accept the next byte to
 *  be transmitted. If the UART is not initialised or handled correctly it
 *  may wait endlessly.
 *
 *  This functions uses spin waiting and hence the respective remarks
 *  for \ref recvSerByte apply.
 *
 *  \param c the character or byte to be sent
 */
void sendSerByte(uint8_t c);

/** Basic UART send multiple bytes from flash
 *
 *  This is a very basic send function for the \ref initUART0 "initialised"
 *  UART. It uses spin waiting and hence the remarks for \ref sendSerByte
 *  apply.
 *
 *  \param src pointer to the first character or byte to be sent in
 *             of the 0-terminated string. Must not be null.
 *  \sa recvSerByte
 *  \sa FAR_ADD
 *  \sa LOW_ADD
 *  \sa sendSerBytes
 */
void sendSerBytes_P(ADDR_T src);

/** Basic UART0 send multiple bytes from RAM
 *
 *  This is a very basic send function for the initialised UART. It uses spin
 *  waiting and hence is meant for situations without any threading support
 *  (or interrupt), as e.g. early initialisation phases or bootloaders.
 *
 *  \param src pointer to the first character or byte to be sent in
 *          of the 0-terminated string, must not be null
 *  \sa recvSerByte
 *  \sa sendSerByte
 *  \sa sendSerBytes_P
 */
void sendSerBytes(char * src);


/** Basic UART wait for a byte received
 *
 *  This is a very basic receive function for the initialised UART. It uses
 *  spin waiting for a received byte timeout of tOut/10 seconds.
 *
 *  Cause of this spin waiting this function is meant for situations without
 *  any threading support (or interrupt), as e.g. early initialisation phases
 *  or bootloaders.
 *
 *  \param tOut a time out in 100 ms (1..254)
 *  \return 0: a byte was received; not 0: timeout
 *  \sa recvSerByte
 */
uint8_t waitSerByte(uint8_t tOut);



/** Basic UART receive one byte
 *
 *  This is a very basic receive function for the \ref initUART0 "initialised"
 *  UART. If no received byte is ready it will endlessly wait for it. To avoid
 *  this it may be guarded by \ref  waitSerByte &mdash; in which case the
 *  (unguarded) macro \ref getSerByte() may also be used.
 *
 *  This function uses spin waiting and would hence be blocking all other
 *  activities. It is meant for situations without any threading support (or
 *  interrupts), as e.g. early initialisation phases or in bootloaders. And
 *  it is, of course, never to be used with an armed watchdog.
 *
 *  \return the character or byte received
 *  \sa sendSerByte
 */
uint8_t recvSerByte(void);

/** Get UART receive error status and flush receiver on error
 *
 *  This function retrieves and returns the error status of an
 *  \ref initUART0 "initialised" UART(0). If no error occurred 0 is returned
 *  and nothing else done.
 *
 *  Otherwise the  \ref initUART0 "initialised" UART's receiver is flushed.
 *
 *  \return 0 on no error or at least one of bits 4 (framing error), 3
 *          (overrun error) or 2 (parity error) is set
 *  \sa recvSerByte
 */
uint8_t recvErrorState(void);

/** Basic function:  UART(0) has one byte received
 *
 *  This expression is non-zero if the UART(0) has one received byte ready
 */
#define isSerByteRec() (UCSR0A & (1 << RXC0))

/** Basic function: UART(0) get one byte already received
 *
 *  This is essentially the unguarded version of \ref recvSerByte(void).
 *  In other words it has to be guarded by \ref waitSerByte(uint8_t tOut) or
 *  by \ref isSerByteRec().
 */
#define getSerByte() UDR0

/** @} */ // Doxygen addtogroup bootIntegr


/**
* \addtogroup bootOper
 * @{
 */

/** The programmers software version
 *
 *  The version is always 2 characters, namely digits. The
 *  \ref PROG_SW_VER "value" is the same for all target platforms.
 */
extern char const programmerSWver[];
#define PROG_SW_VER "10" //!< The \ref programmerSWver \showinitializer

/** The programmers hardware version
 *
 *  The version is always 2 characters, namely digits. The
 *  \ref PROG_HW_VER "value" may depend on the target platform.
 */
extern char const programmerHWver[];

#if defined(PROG_HW_VER) && defined(TIMEOUT_LIGHTSHOW)
 // done elsewhere
#else
# define PROG_HW_VER "00" //!< The \ref programmerHWver \showinitializer
# define TIMEOUT_LIGHTSHOW 0 //!< Time out HMI display \showinitializer
# warning "platform specific bootloader settings undefined"
#endif


/** Software identifier/programmer signature
 *
 *  The signature is always 7 characters. To be accepted by avrDude it has
 *  t start with AVR. The value is \ref PROG_SIGN.
 */
extern char const programmerSign[];
#define PROG_SIGN "AVRBOOT" //!< The \ref programmerSign \showinitializer

/** The bootloader's revision and date
 *
 *  The text starts with blank and ends in a linefeed.
 */
extern char const bootRevDat[];

/** The author of \ref intro_sec "weAutSys" and its bootloader
 *
 *  It is the author's name and his personal domain.
 *
 *  This is a string in (high) flash memory.
 *  \sa copyChars_P()
 */
extern char const  bootAut[];

/** The copyright notice for \ref intro_sec "weAutSys" and its bootloader
 *
 *  This is a string in (high) flash memory.
 *  \sa copyChars_P()
 *  \sa getSomeCharsP(char*, prog_char*, uint8_t)
 */
extern char const  bootCop[];

/** The build date and time
 *
 *  It is the word "build" followed by the date and time where the
 *  C-preprocessor rolled over the (this) file boot109.h. This will be time
 *  where the bootloader was build.
 *
 *  This is a string in (high) flash memory.
 *  \sa copyChars_P()
 */
extern char bootBld[];

#define PARTCODE 0x44 //!< The (outdated) single byte part code \showinitializer

/** Read a flash byte also from high addresses
 *
 *  Depending on the Controller having a \ref LARGE_MEMORY "large flash"
 *  memory or not this macro just delegates to pgm_read_byte_far or
 *  pgm_read_byte_near() respectively.
 * \sa ADDR_T
 */
#if ! LARGE_MEMORY
#  define getFlashByte(addr) pgm_read_byte_near(addr)
#else // not large else large flash memory
#  define getFlashByte(addr) pgm_read_byte_far(addr)
#endif // large flash/

/* BLOCKSIZE should be chosen so that the following holds:
 * BLOCKSIZE * n = PAGESIZE,  where n=1, 2, 3...;
 * N.b.: SPM_PAGESIZE is in bytes
 * */
#define BLOCKSIZE SPM_PAGESIZE

#if defined(GET_LOCK_BITS)    // avr-libc >= 1.2.5
#  define _GET_LOCK_BITS() boot_lock_fuse_bits_get(GET_LOCK_BITS)
#  define _GET_LOW_FUSES() boot_lock_fuse_bits_get(GET_LOW_FUSE_BITS)
#  define _GET_HIGH_FUSES() boot_lock_fuse_bits_get(GET_HIGH_FUSE_BITS)
#  define _GET_EXTENDED_FUSES() boot_lock_fuse_bits_get(GET_EXTENDED_FUSE_BITS)
#endif // defined(GET_LOCK_BITS)

#define _SET_LOCK_BITS(data) boot_lock_bits_set(~data)
#define _ENABLE_RWW_SECTION() boot_rww_enable()

#define _WAIT_FOR_SPM() boot_spm_busy_wait()

#define _FILL_TEMP_WORD(addr,data) boot_page_fill(addr, data)
#define _PAGE_ERASE(addr) boot_page_erase(addr)
#define _PAGE_WRITE(addr) boot_page_write(addr)



/** An eight bit count down display pattern
 *
 *  This is an array in (high!) flash memory. "High" means in the bootloader
 *  flash area. That will be above 64K for some Controllers and means having
 *  to use \c pgm_read_byte_far(farAdd) instead of just
 *  \c pgm_read_byte(add16bit) for access.
 *
 *  For every second counted down there is a 8 bit = 8 LED display
 *  pattern the last one being 0 / all OFF. The total length is 72 (for
 *  72 seconds. For less than 72 seconds start at pattern
 *  &timOutCountPatr[72 -tim]. To start at a certain pattern use
 *  \ref SEC_with_FF (all on), \ref SEC_with_F8 (5 LEDs on) or
 *  \ref SEC_with_F0 (4 LEDs on) for the timeout value \c tim.
 *
 *  \sa bootloaderWlc
 *  \sa FAR_ADD
 */
extern uint8_t const timOutCountPatr[];

#define SEC_with_FF (72-35) //!< timeout value for \ref timOutCountPatr \showinitializer
#define SEC_with_F0 (72-61) //!< timeout value for \ref timOutCountPatr \showinitializer
#define SEC_with_F8 (75-56) //!< timeout value for \ref timOutCountPatr \showinitializer

/** Set the LED
 *
 *  If no test LED is available on the respective platform, this function does
 *  nothing. If more than one LED is available (as with weAut01)
 *  the most prominent red one is used. With weAut01 this is the red status
 *  LED (\ref setStatusLedRd).
 *
 *  \param state  0 / false: turn off; != 0: turn on
 */
void setTheLed(uint8_t state);

/** Hardware criterion for entering bootloader after reset
 *
 *  For weAut_01 this is a pressed enter key while leaving reset (i.e.
 *  releasing the reset key or applying supply).
 */
#ifndef hwCritEntBoot
# define hwCritEntBoot() (0)
# warning "platform hwCritEntBoot() default definition may be wrong"
#endif
// TODO platform specifica to be changed / added here


/**  Send the greeting lines for bootloader's start
 *
 *  This function sends some lines of text over the bootloader's serial link.
 *  These texts  (\ref bootloaderWlc, \ref bootloaderPlatf,
 *  \ref bootRevDat etc.) are to introduce the serial bootloader.
 *
 *  \sa sendSerBytes_P
 *  \sa sendSerBytes
 */
void bootLoaderGreet(void);


/** Write a sequence of words read from UART(0) to flash memory
 *
 *  This is a bootloader support function. It will read size bytes from the
 *  \ref recvSerByte "UART" and transfer them to the program memory starting
 *  at \c address. The \ref ADDR_T "address" parameter must be
 *  a word address. It value incremented by \c size/2 will be returned.
 *
 *  \param size the number of bytes to write to flash memory. It must be
 *              an even number
 *  \param address the start word address to write to in flash memory. (It is
 *              an even byte address.)
 *  \return address + size/2. That is the (incremented) address behind
 *              the last flash word modified
 */
ADDR_T blockLoadFl(uint16_t size, ADDR_T address);

/** Write a sequence of bytes read from UART(0) to EEPROM
 *
 *  This is a bootloader support function. It will read \c size bytes from
 *  the \ref recvSerByte "UART" and transfer them to the EEPROM memory
 *  starting at \c address. The \ref ADDR_T "address" parameter is
 *  a byte address.
 *
 *  \param size the number of bytes to write to EEPROM
 *  \param address the start address (of the first EEPROM byte to be modified).
 *         The parameters's type will fit the flash size even if EEPROMs
 *         are much smaller.
 *  \return address + size. That is the (incremented) address behind
 *              the last EEPROM byte modified
 */
ADDR_T blockLoadEE(uint16_t size, ADDR_T address);



/** Read a sequence of bytes from flash memory and send them via UART(0)
 *
 *  This is a bootloader support function. It will read size bytes from
 *  flash memory starting at \c address and send them as pure binary values
 *  over the \ref sendSerByte "UART".  The \ref ADDR_T "address" parameter
 *  must be a word address. It value incremented by \c size/2 will be returned.
 *
 *  \param size the number of bytes to read from flash or program memory
 *  \param address the start word address to read from flash memory. (It is
 *              an even byte address.)
 *  \return address + size/2. That is the (incremented) address behind
 *              the last flash word read
 */
ADDR_T blockReadFl(uint16_t size, ADDR_T address);


/** Read a sequence of bytes from EEPROM and send them via UART(0)
 *
 *  This is a bootloader support function. It will read size bytes from
 *  EEPROM starting at \c address and send them over the
 *  \ref sendSerByte "UART".
 *
 *  \param size the number of bytes to write to EEPROM memory
 *  \param address pointer to the source memory's start address. It will be
 *         incremented.
 *  \return address + size. That is the (incremented) address behind
 *              the last EEPROM byte read
 */
ADDR_T blockReadEE(uint16_t size, ADDR_T address);

/** Flash memory has no program
 *
 *  This function returns true if the flash cannot have been programmed
 *  with an application software after an erase.
 *
 *  The criterion is simply flash[0] == 0xFFFF. It is, nevertheless, a good
 *  criterion as with this interrupt vector un-programmed there will be no
 *  application.
 */
uint8_t isFlashCleared(void);

/** @} */ // Doxygen addtogroup bootOper
/** @} */ // Doxygen addtogroup bootSup

#endif

/** @} */ // Doxygen addtogroup weAutSys
