// Copyright (c) 2015 Albrecht Weinert, Bochum
// ll_common.h 1 2017-01-25 11:03:06Z albrecht

/**
 * \addtogroup weAutSys
 * @{
 * \addtogroup appLaySup
 * @{
 * \addtogroup comTypes
 * @{
 *
 * \file ll_common.h
 *
 *  \ref intro_sec "weAutSys"' basic common types and helper functions
 *
 *  This file contains the definitions for \ref intro_sec "weAutSys"'
 *  common types and helper macros or functions. They are utilised in more
 *  than one application field respectively application or system module as
 *  well as in the serial bootloader.
 *
 *  This file is part of \ref intro_sec "weAutSys" &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *  <br />It is also used for the \ref bootSup "serial bootloader".
 *
 *  &nbsp; \ref copyLicNote " Copyright &copy; " 2013  Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp;
 *                            <<a href="http://a-weinert.de">a-weinert.de</a>>
 * 1
 * 25.01.2017
 */
//  SVN last change by A. Weinert

/** @} */ // Doxygen addtogroup  comTypes
/** @} */ // Doxygen addtogroup  appLaySup

#ifndef LL_COMMON_H
#define LL_COMMON_H

/* avr/io.h includes (by macro magic; see config.h)
 * CPU specific files:
 atmega644    # takes avr/iom644p.h  + iomxx4.h: weAut, myEthernet
 atmega644p   # takes avr/iom644p.h  + iomxx4.h: weAut, myEthernet
 atmega1284p  # takes avr/iom1284p.h : weAut (NOT + iomxx4 !)
 atmega8      # takes avr\iom8.h     : Dil28 on evaluation Board
 atmega16     # takes avr\iom16.h    : Dil40 on evaluation Board
 atmega328p   # takes avr\iom328p.h  : arduinoUno
 atmega2560   # takes avr\iom2560.h + iomxx0_1.h : (arduinoMega) eval. board
 atmega32     # takes avr\iom32.h    : easyAVR evaluation board
 */
#include <avr/io.h>

#include <stdint.h>
#include "arch/config.h"


// Nota bene: Do check but never set platform settings here.
#ifndef PLATFORM
# error "macro PLATFORM is undefined, setting defaults (check makefile)"
#endif

#if defined(CONFIG_PLATFORM_H)
  // OK: platform specific file could be loaded
#else
# error "no allowed platform specified (check makefile)"
# endif



/** The EEPROM size in byte
 *
 *  \showinitializer
 */
# define EEP_SIZE (E2END + 1)

/**
 * \addtogroup appLaySup
 * @{
 * \addtogroup comTypes
 * @{
 */

//---   tool time macros    ------------------------------------------------

/** @name Defines due to tools
 */
///@{


/** Alternative declaration for flash memory
 *
 *  This macro is a replacement for the use of the PROGMEM macro.
 *
 *  One can end an \c INFLASH declaration by just a (;) semicolon. But as it
 *  is for final flash / program memory variables a declaration is usually
 *  accompanied by an initialiser, like in: \code
 *     INFLASH(char flashString[]) = "in program memory and only there";
 *  \endcode
 *
 *  Most C implementations, like the GNU C Compilers and tools (GCC) used
 *  here, can't handle Harvard architecture in a smooth consistent and
 *  portable way. Instead of healing that, ugly helper constructs
 *  (<avr/pgmspace.h>) have been invented and, alas, must be used. Among other
 *  problems their usage then causes tooling problems (Doxygen, Eclipse).
 *
 *  Internal: In case of macro __DOXYGEN__ defined INFLASH just leaves
 *  \c decl without extra attribute. (Doxygen hates PROGMEM.)
 *
 *  Hint: The pgm_read_byte(address) expression has to be used to access
 *  INFLASEHed final variables. This works only in the range 0..64k-1. If
 *  INFLASEHed items happen to life above 64k
 *  pgm_read_byte_far(longByteAddress) has to be used. longByteAddress will
 *  not be constructed correctly by avr-gcc C pointer handling in most cases.
 *
 *  \param decl a (standard), i.e. "type name", variable declaration
 *          without further modifiers
 *  \hideinitializer
 *  \sa INEEPROM
 */
#ifdef __DOXYGEN__
#  define INFLASH(decl) decl
#else
# define INFLASH(decl) decl PROGMEM
# define STR_EXPAND(tok) #tok
#endif


/** Declaration for EEPROM memory
 *
 *  This macro is used to define variables in the EEPROM usually for
 *  generating individualised .hex files for configuration settings (like
 *  MAC address etc.) If, for example, the source individEEP.c contains
 *  such settings to be in the EEPROM starting at address 0x80 one would
 *  do something like this: \code
       avr-gcc -c -mmcu=atmega1284p -Wl,--section-start=.eeprom=0x0080 -I.  -I ./include individEEP.c
       avr-objcopy -j .eeprom --change-section-lma .eeprom=128 --no-change-warnings -O ihex individEEP.o  individEEP.hex
       avrdude -p atmega1284p -c avrisp2  -P com4   -U eeprom:w:individEEP.hex:i
 \endcode
 *
 *  One can end an \c INEEPROM declaration by just a (;) semicolon. But as it
 *  is in the end for generating final values for (individualised) EEPROM
 *  .hex files the variables defined are usually accompanied
 *  by an initialiser, like in the example: \code
 *     INEEPROM(uint8_t redundantEndMarker) = 0x77;
 *  \endcode
 *
 *  \param decl a (standard), i.e. "type name", variable declaration
 *          without further modifiers
 *  \hideinitializer
 *  \sa  INFLASH
 */
#define INEEPROM(decl) decl __attribute__((section(".eeprom")))


/** A macro value as string
 *
 *  This macro implements the "stringification" of a marco's value. See
 *  gnu.org's GCC documentation (chapter 1.4.4). GCC's stringification has
 *  some special rules concerning white space and escapes. Hence for more
 *  complicated values the result might not meet the expectations.
 *
 *  \param leMac the macro to stringify
 */
#define STR(leMac) STR_EXPAND(leMac)


///@} // end member group macros due to tooling


//--------------  system data  ------------------------------------------


/** The CPU clock frequency (as string)
 *
 *  See the macro \c F_CPU defined in the \c makefile.
 *
 *  This is its value as string, normally "20000000" for 20MHz.
 */
#define FCPU_S STR(F_CPU)

/** The platform name (as string)
 *
 *  See the macro (respectively make variable) \c PLATFORM defined in
 *  the \c makefile.
 *
 *  This is its value as string, that is "weAut_01" and the like.
 */
#define PLATFORM_S STR(PLATFORM)


/** The runtime's author
 *
 *  It is the author's name and his personal domain. <br />
 *  The value is a "quoted" string.
 *  \showinitializer
 */
#define SYST_AUT "Albrecht Weinert <a-weinert.de> "

/** The runtime's copyright notice
 *
 *  The value is a two line "quoted" string with trailing linefeed.
 *  \showinitializer
 */
#define SYST_COP "Copyright (c) 2014  weinert - automation  \n"\
                 "Prof. Dr.-Ing. Albrecht Weinert,  Bochum  \n"
//                0123456789.0123456789.0123456789.0123456789.

/** The runtime's system build and time
 *
 *  It is the time this file was processed by the compiler &mdash; or more
 *  correct by the pre-processor.
 *
 *  The value is a "quoted" string.
 *
 *  \internal This is, partly, a doubtful feature. By including the build date
 *  as (flash) string every build leads to different *.hex files, even in none
 *  else has been modified. The problem is handled by not putting the *.hex
 *  under version control, but an archive /atMegaBootloader/hexNtabs.zip. It
 *  contains the *.hex and the *_ext.tab for all target platforms (hence all
 *  needed to flash and link to the serial bootloader). A provided script
 *  /atMegaBootloader/makeAllBootLoaders.bat will generate all but will
 *  modify the archive  /atMegaBootloader/hexNtabs.zip if and only if sources
 *  or makefiles were changed. Running makeAllBootLoaders.bat before commit
 *  will provide this archive with the revision of the last changed source.
 */
#define SYST_BLD "build " __DATE__ " " __TIME__ " "


//---------------    helper macros   ----------------------------------

/** Get the high byte (of a 16 bit value)
 *
 *  This is a helper. It gets second byte of a value (larger than one byte).
 *  That would be the high byte of a 16 bit value.
 *
 *  It is expressed by masking the high byte and shifting right by 8.
 *  The AVR C Compiler is clever enough to just pick the appropriate
 *  byte respectively register as result. So this helper does not harm.
 */
#define HBYTE(x) (((x) & 0xFF00) >> 8)

/** Get the low byte (of a 16 bit value)
 *
 *  This is a helper that just masks the lowest byte of a 16 bit (or
 *  larger) value.
 *
 *  <b> &nbsp; See also: &nbsp; </b> \ref HBYTE
 */
#define LBYTE(x) ((x) & 0x00FF)


//------------- common constants ---------------------------------------

/** A boolean false respectively off
 *
 *  The value of false / off / aus / teint is 0.
 */
#define OFF 0

/** A boolean true respectively on
 *
 *  The value of true / on / an / dmarr is
 *  \c FF... meaning all bits one for any size or type.
 */
#define ON (~0)


/** The linefeed code
 *
 *  A LF character is usually the equivalent of \\n.
 *  \ref LF and \ref CR are counted in the \ref serCom "serial input"
 *  buffer. <br />
 *  The value of \c LF is 10 respectively 0x0A
 */
#define LF 0x0A

/** The carriage return code
 *
 *  \sa LF
 *  The value of \c CR is 13 respectively 0x0D
 */
#define CR 0x0D

/** The bell code
 *
 *  A BEL character is produces an audible (or visible) signal but no
 *  visible / printable character output (not even white space). <br />
 *  BEL my be used by some \ref telnetServer "Telnet" software. <br />
 *  The value of \c BEL is 7
 */
#define BEL 7

/** The back space code
 *
 *  BS may be used by some \ref telnetServer "Telnet" software to
 *  [rfc854]: <br />
 *  moves the (NVT printer's) print head one character position towards
 *  the left margin. <br />
 *  The value of \c BS is 8
 */
#define BS 8

/** The horizontal tab code
 *
 *  HT may be used by some \ref telnetServer "Telnet" software to
 *  [rfc854] <br />
 *  move the (NVT printer's) print head to the next horizontal tab stop.
 *  It remains unspecified how either party determines or establishes where
 *  such tab stops are located. [rfc854 cite end] <br />
 *  The value of \c HT is 9
 */
#define HT 9

/** The vertical tab code
 *
 *  VT may be used by some \ref telnetServer "Telnet" software to
 *  [rfc854] <br />
 *  move the (NVT printer's) print head to the next vertical tab stop.
 *  It remains unspecified how either party determines or establishes where
 *  such tab stops are located. [rfc854 cite end] <br />
 *  The value of \c VT is 11 (0x0B)
 */
#define VT 11

/** The form feed code
 *
 *  FF may be used by some \ref telnetServer "Telnet" software to
 *  [rfc854] <br />
 *  move the (NVT printer's) print head  to the top of the next page, keeping
 *  the same horizontal position. [rfc854 cite end] <br />
 *  The value of \c FF is 12 (0x0C)
 */
#define FF 12

/** The Escape code
 *
 *  ESC may be used as a sync character e.g. by some bootloader software.
 *
 *  The value of \c ESC is 0x1B
 */
#define ESC 0x1B

//--------------- basic architecture constants -------------------------

/** The (maximum) boot size in bytes
 *
 *  \showinitializer
 */
#if FLASHEND > 0x1F000    // was wrongly 0x3F000  (18.09.2014)
        #define BOOTSIZE 8192
#else
        #define BOOTSIZE 4096 // chg. avec for uno 11.03.14 A.W.
#endif

/** The end of the application flash area (as byte address + 1)
 *
 *  This value is the end of the Read-While-Write (RWW) section in flash + 1
 *  as byte address. For an ATmega 1284 this is e.g.: <br/> &nbsp;
 *  &nbsp; 122880 = 1E000 =  F000 * 2 <br />
 *  For an ATmega 2560 this is e.g.: <br/> &nbsp;
 *  &nbsp; 253952 = 3E000 = 1F000 * 2 <br />
 *
 *  The usage of byte addresses forces \ref LARGE_MEMORY "more than 16 bit"
 *  for the address in this case.
 *
 *  The value is ((\ref BOOTL_BEG "FLASHEND" - \ref BOOTSIZE) + 1)
 */
#define APP_END  ((FLASHEND - BOOTSIZE) + 1)

/** The start address of the bootloader (area)
 *
 *  This byte address in flash memory is just \ref APP_END.
 */
#define BOOTL_BEG APP_END


/** The flash memory byte address type
 *
 *  The ATmega's flash memory is a word (16bit) organised program memory in
 *  Harvard architecture. Hence a 16 bit program counter is sufficient for
 *  a 128 kByte flash (ATmega1284 e.g.). Nevertheless final variables can be
 *  placed in flash, too, forcing it to be byte addressable therefore.
 *  In that sense \c LARGE_MEMORY true (not 0) means we need byte
 *  addresses larger than 16 bit forcing the \ref ADDR_T "address type"
 *  from 16 to 32 bit. <br />
 *  Note: 24 bit would suffice for all large ATmegas, but C hardly handles
 *        that size.
 * \showinitializer
 * \sa ADDR_T
 */
#if FLASHEND > 0x07FFF
#define LARGE_MEMORY 1
#  define ADDR_T uint32_t  //!< flash address type is 32 bit for this Controller
#  define ADDR_U ucnt32_t  //!< flash address type is 32 bit for this Controller
#  define addr_v v32  //!< flash address type is 32 bit for this Controller
#else
#define LARGE_MEMORY 0
#  define ADDR_T uint16_t  //!< flash address is 16 bit for this Controller
#  define ADDR_U ucnt16_t  //!< flash address is 16 bit for this Controller
#  define addr_v v16   //!< flash address is 16 bit for this Controller
#endif



//--------------- common structures -----------------------------------

/** A medium (16 bit) value in different resolutions
 *
 *  This type is for any (non negative) counting or other 16 bit value.<br />
 *  It features 8 and 16 bit resolution as well as direct access to the high
 *  and low byte.<br />
 *
 *  \sa ucnt32_t
 */
typedef union {

/** The value as 16 bit unsigned  */
   uint16_t v16;
/** The (full) value (16 bit unsigned)  */
   uint16_t val;

/** The values as 8 bit unsigned
 *
 *  \c v8[0] would be the 8 bit counter value &mdash; in architecture
 * (Atmel little) endianess. And \c v8[1] would be the high byte.
 *
 *  If \c v16 is used to hold a network (wrong, big) endian value that
 *  would be the other way round, of course.
 */
   uint8_t v8[2];
} ucnt16_t; //--------------------------------------------------


/** A big (32 bit) value in different resolutions
 *
 *  This type is for any (non negative) counting or other 32 bit value or
 *  for use as a result "accumulator", even in "wrong" big endianess. <br />
 *  It features 8, 16 and 32 bit resolution as well as direct access to all
 *  four bytes. <br />
 *
 *  \sa msClock(void)
 *  \sa ucnt16_t
 */
typedef union {

/** The value as 32 bit unsigned  */
   uint32_t v32;
/** The full value (32 bit unsigned)  */
   uint32_t val;

/** The value as 16 bit unsigned
 *
 *  Due to unsigned arithmetic properties it is quite simple to base all
 *  millisecond resolution timers up to ~32s on this resolution.
 */
   uint16_t v16[2];

/** The values as 8 bit unsigned
 *
 *  v8[0] would be the 8 bit counter value  &mdash; in architecture
 *  (atmel little) endianess.
 *
 */
   uint8_t v8[4];

} ucnt32_t;  //--------------------------------------------------


/** Two unsigned bytes intended for quotient and remainder
 *
 *  This structure is used as return type of divide functions, like
 *  \ref divByVal10(uint8_t), \ref divWByVal10toByte(uint16_t) etc. But it can be
 *  used as a common pair of unsigned 8 bit values, as well.
 *  \sa u16div_t
 */
typedef struct {
   uint8_t quot; //<! Usually the quotient: quot = dividend / divisor
   uint8_t rem; //<! Usually the remainder: rem = dividend % divisor
} u8div_t; //--------------------------------------------------


/** Two unsigned words intended for quotient and remainder
 *
 *  This structure is used as return type of divide functions, like
 *  \ref divByVal10(uint8_t), \ref divWByVal10toByte(uint16_t) etc. But it can be
 *  used as a common pair of unsigned 16 bit values, as well.
 *  \sa u8div_t
 */
typedef struct {
   uint16_t quot; //<! Usually the quotient: quot = dividend / divisor
   uint16_t rem; //<! Usually the remainder: rem = dividend % divisor
} u16div_t;


/** @} */ // Doxygen addtogroup comTypes
/** @} */ // Doxygen addtogroup appLaySup

/**
 * \addtogroup llSySer
 * @{
 */


/** Set (boolean or operation) port-bits
 *
 *  These low level helpers would / should hardly be called directly by
 *  user / application software.
 */
#define orAP(mask) PORTA |= (mask)

/** Unset (boolean and operation) port bits
 *
 *  These low level helpers would / should hardly be called directly by
 *  user / application software.
 */
#define andAP(mask) PORTA &= (mask)
// void andAP(uint8_t mask)  __attribute__((always_inline));

/** @copydoc orAP()  */
#define orBP(mask) PORTB |= (mask)
// void orBP(uint8_t mask)  __attribute__((always_inline));

/** @copydoc andAP()  */
#define andBP(mask)  PORTB &= (mask)
// void andBP(uint8_t mask)  __attribute__((always_inline));

/**  Unset and set port B bits */
#define andOrBP(andMask, orMask) PORTB = (PORTB & (andMask)) | (orMask)

/** @copydoc orAP()  */
#define orCP(mask)  PORTC |= (mask)
//  void orCP(uint8_t mask)   __attribute__((always_inline));

/** @copydoc andAP()  */
#define andCP(mask) PORTC &= (mask)
//  void andCP(uint8_t mask)  __attribute__((always_inline));

/**  Unset and set port C bits */
#define andOrCP(andMask, orMask) PORTC = (PORTC & (andMask)) | (orMask)
// void andOrCP(uint8_t andMask, uint8_t orMask)  __attribute__((always_inline));

/** @copydoc orAP()  */
#define orDP(mask)  PORTD |= (mask)
// void orDP(uint8_t mask)   __attribute__((always_inline));

/** @copydoc andAP()  */
#define andDP(mask) PORTD &= (mask)

/** Concatenation helper macro  x ## y */
#define TOKENPASTE(x, y) x ## y

/** Set (boolean or operation) port-bits
 *
 *  This low level helper would / should hardly be called directly by
 *  user / application software.
 *  \param port The port's letter (A, B, C ...)
 *  \param mask The value to be ORed to PORTx
 */
#define orPort(port,mask)  TOKENPASTE(PORT, port)  |= (mask)

/** Set all port-bits
 *
 *  This low level helper would / should hardly be called directly by
 *  user / application software.
 *  \param port The port's letter (A, B, C ...)
 *  \param value The value to be assigned to PORTx
 */
#define setPort(port,value)  TOKENPASTE(PORT, port)  = (value)

/** Unset (boolean and operation) port-bits
 *
 *  This low level helper would / should hardly be called directly by
 *  user / application software.
 *  \param port The port's letter (A, B, C ...)
 *  \param mask The value to be ANDed to PORTx
 */
#define andPort(port,mask) TOKENPASTE(PORT, port)   &= (mask)

#if CANT_OUT2PIN
/** Toggle (boolean xor operation) port-bits
 *
 *  This low level helper would / should hardly be called directly by
 *  user / application software.
 *  \param port The port's letter (A, B, C ...)
 *  \param mask The value to be XORed to PORTx (not by  the PINx trick)
 *  \sa outPins
 */
#define xorPort(port,mask)  TOKENPASTE(PORT, port) = TOKENPASTE(PORT, port) ^ (mask)
#else
/** Toggle (boolean xor operation) port-bits
 *
 *  This low level helper would / should hardly be called directly by
 *  user / application software.
 *  \param port The port's letter (A, B, C ...)
 *  \param mask The value to be XORed to PORTx (by using the PINx trick)
 *  \sa outPins
 */
#define xorPort(port,mask)  TOKENPASTE(PIN, port) = (mask)
#endif

/** Output to a port (by letter)
 *
 *  This low level helper would / should hardly be called directly by
 *  user / application software.
 *  \param port The port's letter (A, B, C ...)
 *  \param mask The value to be set resp. output to PORTx
 *  \sa outPins
 */
#define toPort(port,mask)  TOKENPASTE(PORT, port) = (mask)

/** Re-read output from a port (by letter)
 *
 *  This low level helper would / should hardly be called directly by
 *  user / application software.
 *  \param port The port's letter (A, B, C ...)
 *  \return  this expression is the re-read output from PORTx
 */
#define fromPort(port) TOKENPASTE(PORT, port)

/** Input  port (by letter)
 *
 *  This low level helper would / should hardly be called directly by
 *  user / application software.
 *  \param port The port's letter (A, B, C ...)
 *  \return  this is I/O-Register  PINx
 *  \sa inpPins
 */
#define inPort(port) TOKENPASTE(PIN, port)


/** Set port pins as input.
 *
 *  This is a low level configuration function. It should hardly be
 *  called directly by user / application software (except with good
 *  knowledge and care).
 *  \param port The port's letter (A, B, C ...)
 *  \param mask Bit values of 1 mark the bits reps. pins to be set to input
 *  \sa outPins
 */
#define inpPins(port,mask) TOKENPASTE(DDR, port) &= ~(mask)

/** Set port pins as output.
 *
 *  This is a low level configuration function. It should hardly be
 *  called directly by user / application software (except with good
 *  knowledge and care).
 *  \param port The port's letter (A, B, C ...)
 *  \param mask Bit values of 1 mark the bits reps. pins to be set to input
 *  \sa inpPins
 */
#define outPins(port,mask) TOKENPASTE(DDR, port) |= (mask)

/** Direction port (by letter)
 *
 *  This low level helper would / should hardly be called directly by
 *  user / application software.
 *  \param port The port's letter (A, B, C ...)
 *  \return  this is I/O-Register DDRx
 *  \sa inpPins
 */
#define dirPort(port) TOKENPASTE(DDR, port)


/**
* \addtogroup embDriv
* @{
*/


/** @} */ // Doxygen addtogroup embDriv

/**
 * \addtogroup  sysInit
 * @{
 */


/** Calculate the prescaler setting for a desired baudrate
 *
 *  The calculated and returned \c prescaler divisor setting must be in the
 *  range 0 .. 4096. Otherwise the desired \c baudRate can't even be
 *  approximated with the given processor frequency (e.g. F_CPU=20000000).
 *
 *  The effective baudRate can be calculated by
 *  \ref uartBaud(uint16_t, uint8_t) to check the tolerance/error.
 *
 *  \note This function probably involves 32 bit arithmetic making it a bit
 *  expensive with respect to processor usage. It is intended for use in the
 *  reset / initialisation phase. Instead of calling it regularly with a
 *  limited set of parameter values, it is recommendable to cache those
 *  repeatedly needed results.
 *
 *  \param x2 true: double speed (half receive samples)
 *  \param baudRate the rate desired
 *  \return the UART prescaler setting; it must be 0..4096
 */
uint16_t uartPrescaler(uint32_t baudRate, uint8_t x2);

/** Calculate the true baudrate for a prescaler setting
 *
 *  \note As this function involves 32 bit arithmetic, it is expensive with
 *  respect to processor usage. It is intended for use in the reset /
 *  initialisation phase. Instead of calling it regularly with a limited set
 *  of parameter values, it's recommendable to store results calculated while
 *  initialising for later use.
 *
 *  \param uartPrescal 0..4096 prescaler
 *  \param x2 true: double speed (half receive samples)
 *  \return the baudrate
 *  \sa uartInit
 */
uint32_t uartBaud(uint16_t uartPrescal, uint8_t x2);

/** @} */ // Doxygen addtogroup sysInit
/** @} */ // Doxygen addtogroup llSySer


/**
 * \addtogroup utilHelp
 * @{
 *
 * \addtogroup utilArith
 * @{
 *
 * @name Optimised Divide functions (optionally in bootloader)
 */
///@{

/** Unsigned 16 bit divide
 *
 *  This functions's effect is \code
 *    result (return)  = dividend / divisor;
 *    *rem  = dividend % divisor;
 *  \endcode
 *  As of June 13 this function is more than twice as fast &mdash; and shorter
 *  &mdash; as the code generated by the AVR-GCC C compiler.
 *
 *  Anyway divide is a quite expensive operation for an 8 bit RISC machine
 *  without a divide instruction. Hence the using divide operators (and their
 *  direct replacements e.g. by this function, even though optimised as said)
 *  is discouraged. For special cases use the faster implementations like
 *  \ref  divWByVal1000() &mdash; or, at least, use the function with the
 *  smallest possible operand and result types.
 *  \param rem  pointer to remainder; if null the remainder is lost
 *  \param dividend the number to be divided
 *  \param divisor  the divisor
 *  \return the quotient \c divisor / \c dividend
 *  \sa div24
 *  \sa div32
 */
uint16_t div16(uint16_t * rem, uint16_t dividend, uint16_t divisor);

/** Unsigned 24 bit divide
 *
 *  This functions's effect is \code
 *    result (return)  = dividend / divisor;
 *    *rem  = dividend % divisor;
 *  \endcode
 *  This function is quite similar to \ref div32. It also shares the 32 bit
 *  parameter and return types (as avr-gcc C has no 24 bit type).
 *  Nevertheless it just operates on and returns 24 bit unsigned numbers
 *  &mdash; just ignoring respectively setting 0 the upper byte.
 *
 *  So it is about twice as fast as the optimised \ref div32.
 *
 * \param rem  pointer to the 32 bit variable for the 24 bit remainder;
 *             if null the remainder is lost; upper byte will be 0.
 * \param dividend the 24 bit number to be divided (upper byte ignored)
 * \param divisor  the 24 bit divisor (upper byte ignored)
 * \return the 24 bit quotient \c divisor / \c dividend (upper byte 0)
 * \sa div16
 */
uint32_t div24(uint32_t * rem, uint32_t dividend, uint32_t divisor);

/** Unsigned 32 bit divide
 *
 *  This functions's effect is \code
 *    result (return)  = dividend / divisor;
 *    *rem  = dividend % divisor;
 *  \endcode
 *  As of June 13 this function is more than twice as fast &mdash; and
 *  shorter &mdash; as the code generated by the AVR-GCC C compiler.
 *
 *  32 Bit divide is a very expensive operation for an 8 bit RISC machine
 *  without a divide instruction and only single bit shifts for one
 *  byte. <br />
 *  Hence the using it this division by C operators and % or even this
 *  optimimised replacement function is discouraged. For special cases use the
 *  faster implementations like \ref  divWByVal1000() &mdash; or, at least,
 *  use the function with the smallest possible operand and result types.
 *  \ref div16 e.g. will be four times faster.
 * \param rem  pointer to remainder; if null the remainder is lost
 * \param dividend the number to be divided
 * \param divisor  the divisor
 * \return the quotient \c divisor / \c dividend
 * \sa div16
 * \sa div24
 */
uint32_t div32(uint32_t * rem, uint32_t dividend, uint32_t divisor);

/** Unsigned 32 bit divide by 24 bit
 *
 *  This functions's effect is \code
 *    result (return)  = dividend / divisor;
 *    *rem  = dividend % divisor;
 *  \endcode
 *  This function is quite similar to \ref div32 respectively \ref div24 or a
 *  mix  of both. It shares the 32 bit parameter and return types (as avr-gcc
 *  C has no 24 bit type).
 *
 *  The divisor and remainder are 24 bit numbers (upper 32 bit byte ignored or
 *  set 0). The dividend and the quotient are 32 bit.
 *
 *  It is about 20% faster than \ref div32 (which is quite a lot on a RICS
 *  without division).
 *
 * \param rem  pointer to the 32 bit variable for the 24 bit remainder;
 *             if null the remainder is lost; upper byte will be 0.
 * \param dividend the 32 bit number to be divided
 * \param divisor  the 24 bit divisor (upper 32 bit byte ignored)
 * \return the 32 bit quotient \c divisor / \c dividend (upper byte 0)
 * \sa div16
 */
uint32_t div32by24(uint32_t * rem, uint32_t dividend, uint32_t divisor);

/** Unsigned 32 bit divide by 16 bit
 *
 *  This functions's effect is \code
 *    result (return)  = dividend / divisor;
 *    *rem  = dividend % divisor;
 *  \endcode
 *  This function is quite similar to \ref div32 respectively \ref div16 or a
 *  mix of both.
 *
 *  The divisor and remainder are 16 bit numbers. The dividend and the
 *  quotient are 32 bit.
 *
 *  It is about 35% faster than \ref div32 (which is quite a lot on a RICS
 *  without division).
 *
 * \param rem  pointer to the 16 bit remainder;
 *             if null the remainder is lost
 * \param dividend the 32 bit number to be divided
 * \param divisor  the 16 bit divisor
 * \return the 32 bit quotient \c divisor / \c dividend
 * \sa div24
 */
uint32_t div32by16(uint16_t * rem, uint32_t dividend, uint16_t divisor);

///@} // end member group Divide functions

/** @} */ // Doxygen defgroup utilArith
/** @} */ // Doxygen defgroup utilHelp

#endif /* LL_COMMON_H */
/** @} */ // Doxygen addtogroup weAutSys
