/** \file main.c
 *  The demo application for for Arduino, easyAVR (7) and
 *  further supported AVR platforms. For weAut_01 and weAut_00 weAutSys is
 *  the "demo".
 *
 *  This example / demo program shows how to link an application with
 *  the serial bootloader's library.
 *
 *  <br /> &nbsp;
 * \ref copyLicNote " Copyright &copy; " 2014 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * $Revision: 1 $
 * $Date: 2017-01-25 11:03:06 +0000 (Wed, 25 Jan 2017) $
 */
// Copyright  2013  Albrecht Weinert, Bochum

// next three includes point to elements in the weAutSys' serial bootloader
#include "arch/config.h"
#include "bootloader/boot109.h"
#include "we-aut_sys/ll_common.h"

#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <stddef.h>

uint8_t resetCauses;

/** main's welcome greeting and Copyright */
INFLASH(char const mainWlc[]) = "\n\n\n\n\n\n\n\n\n"
      " * * * *   Welcome to weAut's AVR demo program V0.2   * * * *  \n"
      "                                                               \n"
      " It links to and uses weAutSys' serial bootloader functions.   \n"
      " Copyright (c) 2014  weinert - automation      <weAut.de>      \n"
      " Prof. Dr.-Ing. Albrecht Weinert,  Bochum  <a-weinert.de>      \n"
      "                                                               \n";

/** This little demo program's help text. */
INFLASH(char const helpText[]) =
      " Single letter commands to this little demo program:           \n"
      "                                                               \n"
      "  H ? : this help (+ welcome, version and copyright) text      \n"
      "  G   : the greeting (welcome, version and copyright) alone    \n"
      "  P   : beep (German Pieps) plays one tone (chamber pitch A)   \n"
      "  K   : piano (German Klavier) your keybord is a piano         \n"
      "        qwerttzuio.. is the scale of C major (C-Dur) + above   \n"
      "        cdefgah are the C major tones, shift is one octave up, \n"
      "        23 567 90 are piano's black keys (halftones), blank    \n"
      "        and unused keys are rest (Pause); Esc ends the piano   \n"
      "  Y   : plays Yesterday (by Lennon/McCartney). Hitting a key   \n"
      "        brings you as next tone to (K) the piano. Play on!     \n"
      "  F   : send 0xFF to test the serial link. It ends with any    \n"
      "        byte received serially respectively any key hit.       \n"
      "  B   : enter the serial bootloader and wait for correct       \n"
      "        avrDude (AVR109) communication.                        \n";


#define BUZZ_LOW() andPort(BUZZER_PORT, ~(1 << BUZZER_PIN))
#define BUZZ_TOG() xorPort(BUZZER_PORT, 1 << BUZZER_PIN)
#define BUZZ_HIGH() orPort(BUZZER_PORT, 1 << BUZZER_PIN)

/* Wait multiples of 25 s (Helper function)
 *
 * Used as 1/2 period, of course, we get 50s resp. 20kHz.
 */
void wait25n(uint8_t n){
   for(; --n; ) {
      uint8_t lc =  // 130 for 16MHz
            ((F_CPU * 25 / 1000000) - 5) / 3;  // 25 s; -5 extraLoop
      for(;--lc;){
         asm("");
      } // for lc
#ifdef TEST_PIN0_PORT
      xorPort(TEST_PIN0_PORT, (1<<TEST_PIN0_PIN)); // toggle testPin0 to check 25 s 20KHz
#endif
   } // for n
} // wait25n(n)

/* This macro calculates a divisor to 20kHz for a given frequency x */
#define Hz(x) (20394.6422/x + 0.5) // 20,4kHz measured for wait25n()-toggle

/* The following values pNote define a tone (c, cis etc.) as a
 * "subtone to 20Khz" respectively frequency divider value. */
#define pC   Hz(130.813)
#define pCis Hz(138.591)
#define pD   Hz(146.832)
#define pDis Hz(155.563)
#define pE   Hz(164.814)
#define pF   Hz(174.614)
#define pFis Hz(184.997)
#define pG   Hz(195.998)
#define pGis Hz(207.652)
#define pA   Hz(220)
#define pB   Hz(233.082)
#define pH   Hz(246.942)
#define pc   Hz(261.626)
#define pcis Hz(277.183)
#define pd   Hz(293.665)
#define pdis Hz(311.127)
#define pe   Hz(329.628)
#define pf   Hz(349.228)
#define pfis Hz(369.994)
#define pg   Hz(391.995)
#define pgis Hz(415.305)
#define pa   Hz(440)
#define pb   Hz(466.164)
#define ph   Hz(493.883)

#define pc3   Hz(523.251)
#define pcis3 Hz(554.365)
#define pd3   Hz(587.330)
#define pdis3 Hz(622.254)
#define pe3   Hz(659.255)
#define pf3   Hz(698.456)
#define pfis3 Hz(739.989)
#define pg3   Hz(783.9918)
// intern 21Nov13: all one octave deeper.
// rationale: all notes above 800Hz get to poor precision as 20kHz/div

extern uint8_t const p[];

/* Translation table: Keyboard keys to musical notes
 * a) shift is one octave higher
 * b) qwerrtzuiop+ is one C-Dur-octave + up to f
 * c) 23 567 90 are the half-tones respectively piano black keys
 * d) letters cdefgahb correspond to the respective notes
 * e) German keyboard layout and German standard notation for musical notes:
 *    that is German B = ais = hes = A#  and German H = English B
 */
INFLASH(uint8_t const p[]) = {
      0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,
      0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0, // end control

    // ,   !,    ",     #,      $,     %,   &,    ',
      0,   0, pcis,     0,      0,  pfis, pgis,    0,

   //  (,     ),   *,    +,     ,,    -,    .,     /,
       0,  pcis3, pg3,  pg,     0,    0,    0,    pb,

   //  0,     1,   2,    3,       4,      5,     6,   7,
       pdis,  0,   pCis, pDis,    0,   pFis,  pGis,  pB,

   //   8,    9,   :,    ;,      <,       =,     >,   ?,
       0,  pcis,   0,    0,      0,    pdis3,    0,   0,

   //    @,   A,   B,   C,      D,    E,    F,   G,
         0,  pa,  pb,  pc,     pd,   pe,   pf,  pg,

   //    H,   I,   J,    K,      L,   M,     N,     O,
        ph,  pc3,   0,    0,      0,   0,     0,    pd3,

   //  P,     Q,   R,    S,       T,      U,     V,     W,
       pe3,   pc,  pf,    0,      pg,     ph,     0,    pd,

   //  X,  Y,    Z,    [,         \,    ],    ^,    _,
       0,  0,   pa,    0,      pfis,    0,    0,    0,

    //  `,    a,     b,     c,       d,     e,     f,     g,
        0,   pA,    pB,    pC,      pD,    pE,    pF,    pG,

 //    h,   i,   j,   k,     l,   m,   n,   o,
      pH,   pc,   0,  0,     0,   0,   0,  pd,

 //    p,     q,     r,     s,       t,   u,     v,     w,
      pe,    pC,    pF,     0,      pG,   pH,    0,    pD,

 //    x,    y,     z,    {,        |,     },    ~,    ,
       0,    0,    pA,    0,        0,     0,    0,   0,

      0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,
      0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0, // end control

//                          
      0, 0, 0, 0,  0, 0, 0, pdis,   0, 0, 0, 0,  0, 0, 0, 0,
      0, 0, 0, 0,  0, 0, 0,    0,   0, 0, 0, 0,  0, 0, 0, 0, // nbsp .. /?
//                 
      0, 0, 0, 0,  pfis3 , 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0, // ...
//                                                    
      0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0, pf3, 0, 0, 0, // ..

//                 
      0, 0, 0, 0, pfis, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0, // ...
//                                              
      0, 0, 0, 0,    0, 0, 0, 0,  0, 0, 0, 0,  pf, 0, 0, 0, // ..y:

      0,     0,  };   //   reserve

// Melody definitions (length 1 = eighth note (1/8); that's too coarse)
#define paus(x)  0xFF,x,   //  x * eighth note rest;  x/8 duration
#define ende(x)  0x00,x   // End
#define t(y, d) p##y,d,  // Note with tone y and d/8 duration; d*eighth note
#define n(y) p##y,1,    // eighth note with tone y

uint8_t yesterday[] =  { n(G) n(F) t(F,4) paus(3)
      n(A) n(H) n(cis) n(d)  n(e)  n(f)  n(e) n(d)  n(d)
      paus(2)
      n(d) n(d)  n(c) n(B) n(A) n(G) n(B) n(A) t(A,3)
      n(G) t(F,2) n(A) t(G,3)  n(D) n(F) n(A) t(A,3)
      paus(1) // :|:
      n(G) n(F) t(F,4) paus(3)
      n(A) n(H) n(cis) n(d)  n(e)  n(f)  n(e) n(d)  n(d)
      paus(2)
      n(d) n(d)  n(c) n(B) n(A) n(G) n(B) n(A) t(A,3)
      n(G) t(F,2) n(A) t(G,3)  n(D) n(F) n(A) t(A,3)  // :|:
      t(A,3) t(A,3) n(d)  n(e)  n(f)  n(e) n(d) n(e)
      n(d)  n(c) n(d)  n(A) paus(3)
      t(A,3) t(A,3) n(d)  n(e)  n(f)  n(e) n(d) n(e)
      n(d)  n(c)  n(e)  n(f) n(c)  n(B) n(A)
      n(G) n(F) t(F,2) paus(1)
      n(A) n(H) n(cis) n(d)  n(e)  n(f)  n(e) n(d)  n(d)
      paus(2)
      n(d) n(d)  n(c) n(B) n(A) n(G) n(B) n(A) t(A,3)
      n(G) t(F,2) n(A) t(G,3)  n(D) n(F) n(A) t(A,3)  // :|:
      n(F) n(A) n(G) n(D) n(F) n(A) t(A,4)    ende(0) };

uint8_t tuneA440[] = { t(a,50) ende(0) }; // one tone as (pseudo) melody


/* Internal helper: wait for UART receive with timeout and LED display
 *
 *  \return 0: a byte was received; not 0: timeout
 *  \param sec: 1..72 (!) timeout in seconds
 */
uint8_t waitForUARTrec(uint8_t sec){
#if TIMEOUT_LIGHTSHOW
   ADDR_T patPoint = FAR_ADD(timOutCountPatr[72 - sec]); // pattern pointer
#endif
   while (waitSerByte(5)) {
      setTheLed(0);
      if (! waitSerByte(5)) return 0;
#if   TIMEOUT_LIGHTSHOW
      sec = getFlashByte(patPoint++);
      toHMI8LEDchain(sec);
#else
      --sec;
#endif
      if (!sec) return 1; // time out while waiting for char
      setTheLed(1);
   } // wait for sec s timeout
   return 0; // received
} // waitForUARTrec(uint8_t)


int main(void) __attribute__((OS_main));

/** The application (system) program start
 *
 *  This is the normal system program's program entry.
 *
 *  At present it is a little bit serial (HMI) communication and tone
 *  generator (DO port to loudspeaker) demonstration. With a suitable
 *  terminal program -- like vsst.exe -- one can play piano, start tunes,
 *  enter the serial bootloader etc.
 *
 *  This software's main purpose is to show the linkage to and usage of
 *  bootloader functions and final variables.
 */
int main(void){
   // the bootloader leaves reset causes intact if it wasn't really active
   resetCauses = MCUSR; MCUSR = 0; // save & clear reset cause
   ///  basicSystemInit();  // done in boot loader, already

   sendSerBytes_P(LOW_ADD(mainWlc));  // a greeting (low flash)
   sendSerBytes_P(FAR_ADD(bootloaderPlatf));  // platform info by bootloader

   sendSerBytes_P(FAR_ADD(bootTextReset));  // bootloader function and text
   sendSerBytes_P(resetCauseText_P(resetCauses));
   sendSerByte('\n'); sendSerByte('\n');

   BUZZ_LOW();   setTheLed(0);

   for(;;) { // endless loop
       if (waitForUARTrec(72)) { // timeout while waiting
           sendSerByte('~'); // yes we can (one AC per 72 s timeout)
           continue; // stay in endless loop
       } // timeout while waiting for UART byte

       uint8_t val = getSerByte(); // Get the byte received (one is ready)
       // that is the very simple command line interpreter (CLI)
       // one letter = one command (B H Y P K .. )
       if (val == 'B') {  // B goto bootloader
          bootMain(); // enter the bootloader  (linked bootloader function)
       } // 'B' sends us to the bootloader (until 72s timeout or 'E'  there)


       uint8_t * mel = NULL;  // pointer to a melody to play
       char * byeWords; // pointer to the command's bye bye greeting
       uint8_t halfPer50 = 150;  // variable tone's / note's period (~1/f)

       if (val == 'P') { // P:  beep (German Pieps)
          mel = tuneA440; // one chamber pitch (German Kammerton) A as "melody"
          byeWords = " \n * * end beep 440 Hz nom. \n\n \0";
          goto playTune;
       } // 'P' beeps

       if (val == 'K') { // K Piano (German Klavier)
          byeWords = " \n * * end piano \n\n \0";
          BUZZ_HIGH();   setTheLed(1);

          for (;;) {
             if (halfPer50) { // endless tone or silence
                wait25n(halfPer50);
                BUZZ_TOG(); // toggle beep Port
             } else {
                BUZZ_LOW(); // beep lo
             }
             if (isSerByteRec()) { // ser in
                val = getSerByte();
                if (val == CR && val == LF) continue; // for sake of HTerm
                checkPianoKey:  // bad bad bad: using goto into piano's middle
                BUZZ_LOW(); // beep lo
                if (val < ' ') break;
                wait25(); // piano key click
                halfPer50 = pgm_read_byte(&(p[val]));
             } // // ser in
          } // endless tone or silence
          sendSerBytes(byeWords);
          BUZZ_LOW(); // beep lo
          continue;
       } // 'K' is piano

       if (val == 'Y') { // Y Yesterday (melody)
          mel = yesterday;
          byeWords = " \n * * end Yesterday \n\n \0";
          goto playTune;

          playTune:
          BUZZ_HIGH();   setTheLed(1);

          for(;; ++mel) { // outer loop: through the melody
             BUZZ_LOW(); // beep lo
             uint8_t halfPer50 = *mel;
             uint8_t dur = *(++mel);
             if (!halfPer50) break; // 0 is end
             ucnt32_t duration;
             duration.val = 0;
             duration.v8[2]= dur;  //  << 16;
             duration.val >>= 2; // speed * 4
             for (;;){ // Ton oder Pause
                wait25n(halfPer50);
                if (halfPer50 != 0xFF) BUZZ_TOG(); // toggle beep Port
                if (isSerByteRec()) {
                   val = getSerByte();
                   if (val != CR && val != LF) { // for sake of HTerm
                      sendSerBytes("\n  * * now you play piano ! \n  \0");
                       goto checkPianoKey; // key hit: now go on and ..
                   } // play yourself piano
                }
                duration.val -= halfPer50;
                if (duration.v8[3]) {
                   BUZZ_LOW(); // beep lo
                   wait25(); // piano key click
                   wait25(); // piano key click
                   break;
                }
             }  // Ton oder Pause
          } // outer for: durch die Melodei
          sendSerBytes(byeWords);
          BUZZ_LOW(); // beep lo
          continue;
       } // end melody player (Yesterday and "Pieps" so far)

       if (val == 'F') { // send FF serially (until key received)
          byeWords = " \n * * end sending 0xFF... \n\n \0";
          BUZZ_HIGH();   setTheLed(1);
          BUZZ_LOW(); // beep lo
          for (;;) { // for endless 0xFF
             sendSerByte(0xFF);
             if (isSerByteRec()) { // ser in
                val = getSerByte();
                if (val == CR || val == LF) continue; // for sake of HTerm
                break; // for endless 0xFF
             } // // ser in
          } // endless 0xFF
          sendSerBytes(byeWords);
          BUZZ_LOW(); // beep lo
          continue;
       } // 'F' is serial link test

       if (val == 'H' || val == 'G' || val == '?') { // ? H Help (Hilfe)
          sendSerBytes_P(LOW_ADD(mainWlc));  // a greeting (low flash)
          sendSerBytes_P(FAR_ADD(bootloaderPlatf));  // platform info
          if ( val == 'G' ) continue;
          sendSerByte('\n');
          sendSerBytes_P(LOW_ADD(helpText));
          sendSerByte('\n'); sendSerByte('\n');
          continue;
       } // end CLI help

       // end of CLI interpreted commands all else is just echoed
       sendSerByte(val); // All but B K P Y is just echoed this is a "CLI"
   } // endless for

} // main()


