#include <stdio.h>
#include <windows.h>
#include <signal.h>

/** A very simple serial terminal program -- VSST for short.
 *
 *  This simple Windows program uses the console for input and output to the
 *  serial port. The console output is also used for error and status reports.
 *
 *  The serial communication is full duplex.
 *
 *  VSST sends all keyboard input (except cntlC) immediately to the serial
 *  port end echoes them. "Immediately" means character by character -- and
 *  not by line input as most other terminal programs do.
 *
 *  Translate by:
 *    C:\util\mingw64\bin\gcc.exe vsst.c -o vsst.exe
 *  Run by:
 *    vsst.exe [portName default: COM9 [baudRate default: 38400]]
 *  End by:
 *    cntl-C (^C, 3)
 *
 *  May need:
 *     chcp 1250
 *  before run on some Windows consoles for  etc. to be sent correctly.
 *
 1
 25.01.2017
 */

unsigned char progName[] = "Very simple serial terminal (VSST 0.3)";

LPCSTR portName = "COM9"; // default; may be changed by argument 1
DWORD baudRate = 0; // says default 38400; may be changed by argument 2

/** Helper function: Display (normal) action with program's name. */
void displayAction(const char * action){
   fputs("\n * * ", stdout);
   fputs(progName, stdout);
   fputc(' ', stdout);
   fputs(action, stdout);
   fputs(".    \n", stdout);
} // displayAction(char *)

/** Helper function: Display (normal) report (without program's name). */
void displayReport(const char * report, const char * part2){
   fputs("\n * * ", stdout);
   fputs(report, stdout);
   fputs(part2, stdout);
   fputs(".    \n", stdout);
} // displayReport(char *)

/** Helper function: Display port/handle handling errors. */
void displayPortError(const char * onAction){
   fputs("\n  * * Error ", stdout);
   char buffer [sizeof(long) * 3 + 1];
   ltoa (GetLastError(), buffer, 10);  // GetLastError returns long
   fputs(buffer, stdout);
   fputc(' ', stdout);
   fputs(onAction, stdout);
   fputc(' ', stdout);
   fputs(portName, stdout);
   fputs(" \n", stdout);
} // displayPortError(char *)

/** Read serial bytes to buffer */
DWORD readFromSerialPort(HANDLE hSerial, char * buffer, int buffersize){
   DWORD dwBytesRead = 0;
   if(!ReadFile(hSerial, buffer, buffersize, &dwBytesRead, NULL)){
      return -2; // signals errors
   }
   return dwBytesRead;
} // readFromSerialPort(HANDLE ...)

/** Write serial bytes from buffer */
DWORD writeToSerialPort(HANDLE hSerial, char * data, int length){
   DWORD dwBytesWritten= 0;
   if(!WriteFile(hSerial, data, length, &dwBytesWritten, NULL)){
      return -2;
   }
   return dwBytesWritten;
} // writeToSerialPort(HANDLE ...)


int runOn = 1; // 0 will stop main's inner endless loop
void myHandler(int sig){ runOn = 0; }  // to catch terminate signal (by ^C)

/** The terminal application. */
int main(int argC, char* argV[]){
   displayAction("says Hello");
   if (argC >= 2) {
      portName = argV[1];
      if (argC >= 3) baudRate = atol(argV[2]);
   }

   HANDLE hSerial = CreateFile(portName,
         GENERIC_READ | GENERIC_WRITE,
         0, NULL, OPEN_EXISTING,
         // FILE_ATTRIBUTE_NORMAL |
         //  FILE_FLAG_OVERLAPPED |
         FILE_FLAG_NO_BUFFERING,
         NULL);
   if (hSerial == INVALID_HANDLE_VALUE) {
      displayPortError("on opening");
      return 1;
   }

   DCB dcbSerialParams = { 0 };
   dcbSerialParams.DCBlength = sizeof(dcbSerialParams);
   if (!GetCommState(hSerial, &dcbSerialParams)) {
      displayPortError("on getting state from");
      return 2;
   }

   dcbSerialParams.BaudRate = baudRate ? baudRate : 38400;
   dcbSerialParams.ByteSize = 8;
   dcbSerialParams.StopBits = ONESTOPBIT;
   dcbSerialParams.Parity   = NOPARITY;
   if(!SetCommState(hSerial, &dcbSerialParams)){
      displayPortError("on setting mode of");
      return 3;
   }

   COMMTIMEOUTS timeouts = { 0 };
   timeouts.ReadIntervalTimeout        = 30;  // was 50
   timeouts.ReadTotalTimeoutConstant   = 150; // was 50
   timeouts.ReadTotalTimeoutMultiplier = 0;   // was 10
   timeouts.WriteTotalTimeoutConstant  = 50;
   timeouts.WriteTotalTimeoutMultiplier= 1;   // was 10
   if(!SetCommTimeouts(hSerial, &timeouts)) {
      displayPortError("on setting timeouts of");
      return 4;
   }

   displayReport("Type cntlC to quit VSST; all else is sent to ", portName);
   signal(SIGTERM, myHandler);
   signal(SIGINT, myHandler);

   unsigned char buf[391];
   putchar('\n');
   while(runOn) {
      if (kbhit()) { // poll keyboard
         unsigned int ch =  _getch();
         if (ch == 3) break;
         putchar(ch); // Echo of input
         buf[0] = (unsigned char)ch;
         writeToSerialPort(hSerial, buf, 1);
      } // poll keyboard
      DWORD recB = readFromSerialPort(hSerial, buf, 390); // try to read
      if (recB > 0) { // something came in via serial port
         buf[recB] = 0;
         fputs(buf, stdout);
      } // was serial in
   } // while not cntlC
   putchar('\n');

   CloseHandle(hSerial);
   displayAction("says Goodbye");
   return 0;
} // main(int argC, char *[])
